@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -224,7 +224,7 @@ void insert_accounting_structure(struct primitives_ptrs *prim_ptrs)
         }
 
         memset(elem_acc->clbgp, 0, clb_size);
-        pkt_to_cache_legacy_bgp_primitives(elem_acc->clbgp, plbgp, config.what_to_count, config.what_to_count_2);
+        pkt_to_cache_legacy_bgp_primitives(elem_acc->clbgp, plbgp, config.what_to_count, config.what_to_count_2, config.what_to_count_3);
       }
       else free_cache_legacy_bgp_primitives(&elem_acc->clbgp);
 
@@ -370,7 +370,7 @@ void insert_accounting_structure(struct primitives_ptrs *prim_ptrs)
           exit_gracefully(1);
         }
         memset(elem_acc->clbgp, 0, clb_size);
-        pkt_to_cache_legacy_bgp_primitives(elem_acc->clbgp, plbgp, config.what_to_count, config.what_to_count_2);
+        pkt_to_cache_legacy_bgp_primitives(elem_acc->clbgp, plbgp, config.what_to_count, config.what_to_count_2, config.what_to_count_3);
       }
       else elem_acc->clbgp = NULL;
 
