@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -22,8 +22,13 @@
 #define AMQP_PLUGIN_COMMON_H
 
 /* includes */
+#if defined WITH_RABBITMQ_13
+#include <rabbitmq-c/amqp.h>
+#include <rabbitmq-c/tcp_socket.h>
+#else
 #include <amqp.h>
 #include <amqp_tcp_socket.h>
+#endif
 #include <sys/poll.h>
 
 #include "plugin_common.h"
