@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -54,7 +54,7 @@ char *std_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
 char *lrg_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
 struct bgp_comm_range peer_src_as_ifrange; 
 struct bgp_comm_range peer_src_as_asrange; 
-u_int32_t (*bgp_route_info_modulo)(struct bgp_peer *, path_id_t *, int);
+u_int32_t (*bgp_route_info_modulo)(struct bgp_peer *, rd_t *, path_id_t *, struct bgp_msg_extra_data *, int);
 struct bgp_rt_structs inter_domain_routing_dbs[FUNC_TYPE_MAX], *bgp_routing_db;
 struct bgp_misc_structs inter_domain_misc_dbs[FUNC_TYPE_MAX], *bgp_misc_db;
 bgp_tag_t bgp_logdump_tag;
@@ -298,6 +298,8 @@ void skinny_bgp_daemon_online()
 
   if (config.bgp_table_per_peer_hash == BGP_ASPATH_HASH_PATHID)
     bgp_route_info_modulo = bgp_route_info_modulo_pathid; 
+  else if (config.bgp_table_per_peer_hash == BGP_ASPATH_HASH_MPLSVPNRD)
+    bgp_route_info_modulo = bgp_route_info_modulo_mplsvpnrd;
   else {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unknown 'bgp_table_per_peer_hash' value. Terminating thread.\n", config.name, bgp_misc_db->log_str);
     exit_gracefully(1);
@@ -396,12 +398,21 @@ void skinny_bgp_daemon_online()
 
   {
     char srv_string[INET6_ADDRSTRLEN];
+    char *srv_interface = NULL, default_interface[] = "all";
     struct host_addr srv_addr;
     u_int16_t srv_port;
 
+    if (!config.bgp_daemon_interface) {
+      srv_interface = default_interface;
+    }
+    else {
+      srv_interface = config.bgp_daemon_interface;
+    }
+
     sa_to_addr((struct sockaddr *)&server, &srv_addr, &srv_port);
     addr_to_str(srv_string, &srv_addr);
-    Log(LOG_INFO, "INFO ( %s/%s ): waiting for BGP data on %s:%u\n", config.name, bgp_misc_db->log_str, srv_string, srv_port);
+    Log(LOG_INFO, "INFO ( %s/%s ): waiting for BGP data on interface=%s ip=%s port=%u/tcp\n",
+	config.name, bgp_misc_db->log_str, srv_interface, srv_string, srv_port);
   }
 
   /* Preparing ACL, if any */
@@ -613,6 +624,29 @@ void skinny_bgp_daemon_online()
     sigaddset(&signal_set, SIGINT);
   }
 
+  if (!bgp_misc_db->is_thread) {
+#ifdef WITH_REDIS
+    if (config.bgp_bmp_daemon_ha) {
+      /* Signals for BMP-BGP-HA feature */
+      sigaddset(&signal_set, SIGRTMIN);
+      sigaddset(&signal_set, SIGRTMIN + 1);
+      sigaddset(&signal_set, SIGRTMIN + 2);
+      sigaddset(&signal_set, SIGRTMIN + 3);
+    }
+#endif
+  }
+
+  if (config.dry_run == DRY_RUN_SETUP) {
+    if (!bgp_misc_db->is_thread) {
+      sleep(DEFAULT_SLOTH_SLEEP_TIME); /* Make sure all comes up */
+      Log(LOG_INFO, "INFO ( %s/%s ): Dry run 'setup'. Exiting ..\n", config.name, bgp_misc_db->log_str);
+      exit(0);
+    }
+    else {
+      return;
+    }
+  }
+
   for (;;) {
     select_again:
 
@@ -719,22 +753,19 @@ void skinny_bgp_daemon_online()
 	  bgp_peer_log_seq_init(&bgp_misc_db->log_seq);
       }
 
-
-      int refreshTimePerSlot = config.bgp_table_dump_refresh_time / config.bgp_table_dump_time_slots;
       if (bgp_misc_db->dump_backend_methods) {
+        bgp_misc_db->dump.period = config.bgp_table_dump_refresh_time / config.bgp_table_dump_time_slots;
 	while (bgp_misc_db->log_tstamp.tv_sec > dump_refresh_deadline) {
 	  bgp_misc_db->dump.tstamp.tv_sec = dump_refresh_deadline;
 	  bgp_misc_db->dump.tstamp.tv_usec = 0;
 	  compose_timestamp(bgp_misc_db->dump.tstamp_str, SRVBUFLEN, &bgp_misc_db->dump.tstamp, FALSE,
 			    config.timestamps_since_epoch, config.timestamps_rfc3339, config.timestamps_utc);
-	  bgp_misc_db->dump.period = refreshTimePerSlot;
 
 	  if (bgp_peer_log_seq_has_ro_bit(&bgp_misc_db->log_seq))
 	    bgp_peer_log_seq_init(&bgp_misc_db->log_seq);
 
 	  bgp_handle_dump_event(max_peers_idx);
-
-	  dump_refresh_deadline += refreshTimePerSlot;
+          dump_refresh_deadline += bgp_misc_db->dump.period;
 	}
       }
 
@@ -805,7 +836,7 @@ void skinny_bgp_daemon_online()
 	  */
           if (bgp_batch_is_admitted(&bp_batch, now)) {
             peer = &peers[peers_idx];
-            if (bgp_peer_init(peer, FUNC_TYPE_BGP)) peer = NULL;
+            if (bgp_peer_init(peer, FUNC_TYPE_BGP, BGP_BUFFER_SIZE)) peer = NULL;
 	    else recalc_fds = TRUE;
 
             log_notification_unset(&log_notifications.bgp_peers_throttling);
