@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -64,7 +64,8 @@
 #define BGP_ATTR_AS4_AGGREGATOR                 18
 #define BGP_ATTR_AS_PATHLIMIT                   21
 #define BGP_ATTR_AIGP				26
-#define BGP_ATTR_LARGE_COMMUNITIES		32 /* rfc8092 */
+#define BGP_ATTR_LARGE_COMMUNITIES		32
+#define BGP_ATTR_OTC				35 /* rfc9234 */
 #define BGP_ATTR_PREFIX_SID			40
 
 /* BGP4 internal bitmap type codes.  */
@@ -292,7 +293,7 @@ struct bgp_misc_structs {
   int table_per_peer_buckets;
   int table_attr_hash_buckets;
   int table_per_peer_hash;
-  u_int32_t (*route_info_modulo)(struct bgp_peer *, path_id_t *, int);
+  u_int32_t (*route_info_modulo)(struct bgp_peer *, rd_t *, path_id_t *, struct bgp_msg_extra_data *, int);
   struct bgp_peer *(*bgp_lookup_find_peer)(struct sockaddr *, struct xflow_status_entry *, u_int16_t, int);
   int (*bgp_lookup_node_match_cmp)(struct bgp_info *, struct node_match_cmp_term2 *);
 
@@ -343,7 +344,7 @@ struct bgp_attr {
   struct ecommunity *ecommunity;
   struct lcommunity *lcommunity;
   unsigned long refcnt;
-  u_int32_t flag;
+  u_int8_t rpki_maxlen;
   struct in_addr nexthop;
   struct host_addr mp_nexthop;
   u_int32_t med;
@@ -416,7 +417,7 @@ extern char *std_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
 extern char *lrg_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
 extern struct bgp_comm_range peer_src_as_ifrange; 
 extern struct bgp_comm_range peer_src_as_asrange; 
-extern u_int32_t (*bgp_route_info_modulo)(struct bgp_peer *, path_id_t *, int);
+extern u_int32_t (*bgp_route_info_modulo)(struct bgp_peer *, rd_t *, path_id_t *, struct bgp_msg_extra_data *, int);
 
 extern struct bgp_rt_structs inter_domain_routing_dbs[FUNC_TYPE_MAX], *bgp_routing_db;
 extern struct bgp_misc_structs inter_domain_misc_dbs[FUNC_TYPE_MAX], *bgp_misc_db;
