@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2024 by Paolo Lucente
 */
 
 /*
@@ -28,7 +28,7 @@
 #include "pmbgpd.h"
 #include "rpki/rpki.h"
 
-void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
+void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type, struct bgp_lookup_info *bl_info)
 {
   struct bgp_misc_structs *bms;
   struct bgp_rt_structs *inter_domain_routing_db;
@@ -105,12 +105,21 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       memcpy(&peer_dst_ip, &pbgp.peer_dst_ip, sizeof(struct host_addr));
     }
 
+    /* Get RD for correlation */
     if (pptrs->bitr) {
-      safi = SAFI_MPLS_VPN;
-      memcpy(&rd, &pptrs->bitr, sizeof(rd));
+      rd_t *rd_ptr = (rd_t *) &pptrs->bitr;
+ 
+      if (!rd_ptr->as && !rd_ptr->val && bgp_rd_origin_get(rd_ptr->type) == RD_ORIGIN_MAP) {
+	safi = SAFI_UNICAST;
+	memset(&rd, 0, sizeof(rd));
+      }
+      else {
+	safi = SAFI_MPLS_VPN;
+	memcpy(&rd, &pptrs->bitr, sizeof(rd));
 
-      if (type <= RD_ORIGIN_FUNC_TYPE_MAX) {
-	bgp_rd_origin_set(&rd, lookup_type_to_bgp_rd_origin[type]);
+	if (type <= RD_ORIGIN_FUNC_TYPE_MAX) {
+	  bgp_rd_origin_set(&rd, lookup_type_to_bgp_rd_origin[type]);
+	}
       }
     }
 
@@ -178,61 +187,187 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       }
     }
     else if (pptrs->l3_proto == ETHERTYPE_IPV6) {
-      if (!pptrs->bgp_src) {
-        memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
-        nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
-	nmct2.afi = AFI_IP6;
-	nmct2.safi = safi;
-        nmct2.rd = &rd;
-        nmct2.peer_dst_ip = NULL;
 
-        memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_src, sizeof(struct in6_addr));
-	bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
-		            &pref6, (struct bgp_peer *) pptrs->bgp_peer,
-		            bms->route_info_modulo,
-		            bms->bgp_lookup_node_match_cmp, &nmct2,
-		            bms->bnv, &result, &info);
-      }
+      /* SRv6 Tunnel with IPv4 inner/customer layer */
+      if (pptrs->flow_type.traffic_type == PM_FTYPE_SRV6_IPV4) { 
+        if (!pptrs->bgp_src) {
+          memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+          nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+          nmct2.afi = AFI_IP;
+          nmct2.safi = safi;
+          nmct2.rd = &rd;
+          nmct2.peer_dst_ip = NULL;
 
-      if (!pptrs->bgp_src_info && result) {
-        pptrs->bgp_src = (char *) result;
-        pptrs->bgp_src_info = (char *) info;
-        if (result->p.prefixlen >= pptrs->lm_mask_src) {
-          pptrs->lm_mask_src = result->p.prefixlen;
-          pptrs->lm_method_src = NF_NET_BGP;
+          memcpy(&pref4, &bl_info->inner_ip_src, sizeof(struct in_addr));
+          bgp_node_match_ipv4(inter_domain_routing_db->rib[AFI_IP][safi],
+                              &pref4, (struct bgp_peer *) pptrs->bgp_peer,
+                              bms->route_info_modulo,
+                              bms->bgp_lookup_node_match_cmp, &nmct2,
+                              bms->bnv, &result, &info);
+        }
 
-	  if (config.rpki_roas_file || config.rpki_rtr_cache) {
-	    pptrs->src_roa = rpki_vector_prefix_lookup(bms->bnv); 
-	  }
+        if (!pptrs->bgp_src_info && result) {
+          pptrs->bgp_src = (char *) result;
+          pptrs->bgp_src_info = (char *) info;
+          if (result->p.prefixlen >= pptrs->lm_mask_src) {
+            pptrs->lm_mask_src = result->p.prefixlen;
+            pptrs->lm_method_src = NF_NET_BGP;
+
+            if (config.rpki_roas_file || config.rpki_rtr_cache) {
+              pptrs->src_roa = rpki_vector_prefix_lookup(bms->bnv); 
+            }
+          }
+        }
+
+        if (!pptrs->bgp_dst) {
+          memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+          nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+          nmct2.afi = AFI_IP;
+          nmct2.safi = safi;
+          nmct2.rd = &rd;
+          nmct2.peer_dst_ip = &peer_dst_ip;
+
+          memcpy(&pref4, &bl_info->inner_ip_dst, sizeof(struct in_addr));
+          bgp_node_match_ipv4(inter_domain_routing_db->rib[AFI_IP][safi],
+                              &pref4, (struct bgp_peer *) pptrs->bgp_peer,
+                              bms->route_info_modulo,
+                              bms->bgp_lookup_node_match_cmp, &nmct2,
+                              bms->bnv, &result, &info);
+        }
+
+        if (!pptrs->bgp_dst_info && result) {
+          pptrs->bgp_dst = (char *) result;
+          pptrs->bgp_dst_info = (char *) info;
+          if (result->p.prefixlen >= pptrs->lm_mask_dst) {
+            pptrs->lm_mask_dst = result->p.prefixlen;
+            pptrs->lm_method_dst = NF_NET_BGP;
+
+            if (config.rpki_roas_file || config.rpki_rtr_cache) {
+              pptrs->dst_roa = rpki_vector_prefix_lookup(bms->bnv);
+            }
+          }
         }
       }
 
-      if (!pptrs->bgp_dst) {
-        memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
-        nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
-	nmct2.afi = AFI_IP6;
-	nmct2.safi = safi;
-        nmct2.rd = &rd;
-        nmct2.peer_dst_ip = &peer_dst_ip;
+      /* SRv6 Tunnel with IPv6 inner/customer layer */
+      else if (pptrs->flow_type.traffic_type == PM_FTYPE_SRV6_IPV6) {
+        if (!pptrs->bgp_src) {
+          memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+          nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+          nmct2.afi = AFI_IP6;
+          nmct2.safi = safi;
+          nmct2.rd = &rd;
+          nmct2.peer_dst_ip = NULL;
 
-        memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_dst, sizeof(struct in6_addr));
-	bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
-	     		    &pref6, (struct bgp_peer *) pptrs->bgp_peer,
-			    bms->route_info_modulo,
-			    bms->bgp_lookup_node_match_cmp, &nmct2,
-			    bms->bnv, &result, &info);
-      }
+          memcpy(&pref6, &bl_info->inner_ipv6_src, sizeof(struct in6_addr));
+          bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
+                              &pref6, (struct bgp_peer *) pptrs->bgp_peer,
+                              bms->route_info_modulo,
+                              bms->bgp_lookup_node_match_cmp, &nmct2,
+                              bms->bnv, &result, &info);
+        }
 
-      if (!pptrs->bgp_dst_info && result) {
-        pptrs->bgp_dst = (char *) result;
-        pptrs->bgp_dst_info = (char *) info;
-        if (result->p.prefixlen >= pptrs->lm_mask_dst) {
-          pptrs->lm_mask_dst = result->p.prefixlen;
-          pptrs->lm_method_dst = NF_NET_BGP;
+        if (!pptrs->bgp_src_info && result) {
+          pptrs->bgp_src = (char *) result;
+          pptrs->bgp_src_info = (char *) info;
+          if (result->p.prefixlen >= pptrs->lm_mask_src) {
+            pptrs->lm_mask_src = result->p.prefixlen;
+            pptrs->lm_method_src = NF_NET_BGP;
 
-	  if (config.rpki_roas_file || config.rpki_rtr_cache) {
-	    pptrs->dst_roa = rpki_vector_prefix_lookup(bms->bnv);
-	  }
+            if (config.rpki_roas_file || config.rpki_rtr_cache) {
+              pptrs->src_roa = rpki_vector_prefix_lookup(bms->bnv); 
+            }
+          }
+        }
+
+        if (!pptrs->bgp_dst) {
+          memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+          nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+          nmct2.afi = AFI_IP6;
+          nmct2.safi = safi;
+          nmct2.rd = &rd;
+          nmct2.peer_dst_ip = &peer_dst_ip;
+
+          memcpy(&pref6, &bl_info->inner_ipv6_dst, sizeof(struct in6_addr));
+          bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
+                              &pref6, (struct bgp_peer *) pptrs->bgp_peer,
+                              bms->route_info_modulo,
+                              bms->bgp_lookup_node_match_cmp, &nmct2,
+                              bms->bnv, &result, &info);
+        }
+
+        if (!pptrs->bgp_dst_info && result) {
+          pptrs->bgp_dst = (char *) result;
+          pptrs->bgp_dst_info = (char *) info;
+          if (result->p.prefixlen >= pptrs->lm_mask_dst) {
+            pptrs->lm_mask_dst = result->p.prefixlen;
+            pptrs->lm_method_dst = NF_NET_BGP;
+
+            if (config.rpki_roas_file || config.rpki_rtr_cache) {
+              pptrs->dst_roa = rpki_vector_prefix_lookup(bms->bnv);
+            }
+          }
+        }
+      } 
+
+      /* Plain IPv6 or other IPv6 scenarios */
+      else {
+        if (!pptrs->bgp_src) {
+          memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+          nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+          nmct2.afi = AFI_IP6;
+          nmct2.safi = safi;
+          nmct2.rd = &rd;
+          nmct2.peer_dst_ip = NULL;
+
+          memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_src, sizeof(struct in6_addr));
+          bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
+                              &pref6, (struct bgp_peer *) pptrs->bgp_peer,
+                              bms->route_info_modulo,
+                              bms->bgp_lookup_node_match_cmp, &nmct2,
+                              bms->bnv, &result, &info);
+        }
+
+        if (!pptrs->bgp_src_info && result) {
+          pptrs->bgp_src = (char *) result;
+          pptrs->bgp_src_info = (char *) info;
+          if (result->p.prefixlen >= pptrs->lm_mask_src) {
+            pptrs->lm_mask_src = result->p.prefixlen;
+            pptrs->lm_method_src = NF_NET_BGP;
+
+            if (config.rpki_roas_file || config.rpki_rtr_cache) {
+              pptrs->src_roa = rpki_vector_prefix_lookup(bms->bnv); 
+            }
+          }
+        }
+
+        if (!pptrs->bgp_dst) {
+          memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+          nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+          nmct2.afi = AFI_IP6;
+          nmct2.safi = safi;
+          nmct2.rd = &rd;
+          nmct2.peer_dst_ip = &peer_dst_ip;
+
+          memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_dst, sizeof(struct in6_addr));
+          bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
+                              &pref6, (struct bgp_peer *) pptrs->bgp_peer,
+                              bms->route_info_modulo,
+                              bms->bgp_lookup_node_match_cmp, &nmct2,
+                              bms->bnv, &result, &info);
+        }
+
+        if (!pptrs->bgp_dst_info && result) {
+          pptrs->bgp_dst = (char *) result;
+          pptrs->bgp_dst_info = (char *) info;
+          if (result->p.prefixlen >= pptrs->lm_mask_dst) {
+            pptrs->lm_mask_dst = result->p.prefixlen;
+            pptrs->lm_method_dst = NF_NET_BGP;
+
+            if (config.rpki_roas_file || config.rpki_rtr_cache) {
+              pptrs->dst_roa = rpki_vector_prefix_lookup(bms->bnv);
+            }
+          }
         }
       }
     }
@@ -377,10 +512,11 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
   }
 
   if (nh_peer) {
-    modulo = bms->route_info_modulo(nh_peer, NULL, bms->table_per_peer_buckets);
+    modulo = bms->route_info_modulo(nh_peer, NULL, NULL, NULL, bms->table_per_peer_buckets);
 
     // XXX: to be optimized 
     if (bms->table_per_peer_hash == BGP_ASPATH_HASH_PATHID) modulo_max = bms->table_per_peer_buckets;
+    else if (bms->table_per_peer_hash == BGP_ASPATH_HASH_MPLSVPNRD) modulo_max = bms->table_per_peer_buckets;
     else modulo_max = 1;
 
     memset(&ch, 0, sizeof(ch));
@@ -629,7 +765,7 @@ int bgp_lookup_node_vector_unicast(struct prefix *p, struct bgp_peer *peer, stru
 }
 
 void pkt_to_cache_legacy_bgp_primitives(struct cache_legacy_bgp_primitives *c, struct pkt_legacy_bgp_primitives *p,
-					pm_cfgreg_t what_to_count, pm_cfgreg_t what_to_count_2)
+					pm_cfgreg_t what_to_count, pm_cfgreg_t what_to_count_2, pm_cfgreg_t what_to_count_3)
 {
   if (c && p) {
     if (what_to_count & COUNT_STD_COMM) {
@@ -789,7 +925,7 @@ void free_cache_legacy_bgp_primitives(struct cache_legacy_bgp_primitives **c)
   }
 }
 
-u_int32_t bgp_route_info_modulo_pathid(struct bgp_peer *peer, path_id_t *path_id, int per_peer_buckets)
+u_int32_t bgp_route_info_modulo_pathid(struct bgp_peer *peer, rd_t *rd, path_id_t *path_id, struct bgp_msg_extra_data *bmed, int per_peer_buckets)
 {
   struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   path_id_t local_path_id = 1;
@@ -801,6 +937,18 @@ u_int32_t bgp_route_info_modulo_pathid(struct bgp_peer *peer, path_id_t *path_id
           (bms->table_peer_buckets * per_peer_buckets));
 }
 
+u_int32_t bgp_route_info_modulo_mplsvpnrd(struct bgp_peer *peer, rd_t *rd, path_id_t *path_id, struct bgp_msg_extra_data *bmed, int per_peer_buckets)
+{
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
+  u_int16_t local_rd = 0;
+
+  if (rd) local_rd = (rd->type + rd->as + rd->val);
+
+  return (((peer->fd * per_peer_buckets) +
+          (local_rd % per_peer_buckets)) %
+          (bms->table_peer_buckets * per_peer_buckets));
+}
+
 int bgp_lg_daemon_ip_lookup(struct bgp_lg_req_ipl_data *req, struct bgp_lg_rep *rep, int type)
 {
   struct bgp_misc_structs *bms;
@@ -855,7 +1003,7 @@ int bgp_lg_daemon_ip_lookup(struct bgp_lg_req_ipl_data *req, struct bgp_lg_rep *
 
     if (l3_proto == ETHERTYPE_IP) {
       bgp_node_match(inter_domain_routing_db->rib[AFI_IP][safi],
-			    &req->pref, peer, bgp_route_info_modulo_pathid,
+			    &req->pref, peer, bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
 			    NULL, &result, &info);
 
@@ -867,7 +1015,7 @@ int bgp_lg_daemon_ip_lookup(struct bgp_lg_req_ipl_data *req, struct bgp_lg_rep *
     }
     else if (l3_proto == ETHERTYPE_IPV6) {
       bgp_node_match(inter_domain_routing_db->rib[AFI_IP6][safi],
-		            &req->pref, peer, bgp_route_info_modulo_pathid,
+		            &req->pref, peer, bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
 			    NULL, &result, &info);
 
