@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -304,11 +304,12 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		    	  config.name, bms->log_str, bgp_peer_str, cap_type, ntohs(cap_data.afi), cap_data.safi,
 			  cap_data.sndrcv);
 		    }
-		    if ((cap_data.sndrcv == 2 /* send */) || (cap_data.sndrcv == 3 /* send and receive */)) {
+		    if ((!online && cap_data.sndrcv /* any */) || (online && 
+		       ((cap_data.sndrcv == 2 /* send */) || (cap_data.sndrcv == 3 /* send and receive */)))) {
 		      afi_t cap_add_paths_afi = ntohs(cap_data.afi);
 
 		      if (cap_add_paths_afi < AFI_MAX && cap_data.safi < SAFI_MAX) {
-		        peer->cap_add_paths.cap[cap_add_paths_afi][cap_data.safi] = TRUE;
+		        peer->cap_add_paths.cap[cap_add_paths_afi][cap_data.safi] = cap_data.sndrcv; /* 1, 2 or 3 */
 
 		        if (cap_add_paths_afi > peer->cap_add_paths.afi_max) {
 		          peer->cap_add_paths.afi_max = cap_add_paths_afi;
@@ -347,6 +348,29 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		  bgp_open_cap_reply_ptr += cap_len+2;
 		}
 	      }
+              /* Support for Extended Next-Hop Encoding as of RFC8950, required
+                 for enabling advertising IPv4 NLRI with an IPv6 Next Hop. */
+              else if (cap_type == BGP_CAPABILITY_EXTENDED_NEXT_HOP_ENCODING) {
+                char *cap_ptr = optcap_ptr+2;
+                struct capability_ext_nh_enc_data cap_data;
+
+                for (int cap_idx = 0; cap_idx < cap_len; cap_idx += sizeof(cap_data)) {
+                  memcpy(&cap_data, cap_ptr+cap_idx, sizeof(cap_data));
+
+                  if (online) {
+                    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+                    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: Extended Next Hop Encoding [%u] "
+                                   "AFI [%u] SAFI [%u] Next Hop AFI [%u]\n",
+                                   config.name, bms->log_str, bgp_peer_str, cap_type,
+                                   ntohs(cap_data.afi), ntohs(cap_data.safi), ntohs(cap_data.nh_afi));
+                  }
+                }
+
+                if (online) {
+                  memcpy(bgp_open_cap_reply_ptr, optcap_ptr, cap_len+2);
+                  bgp_open_cap_reply_ptr += cap_len+2;
+                }
+              }
 
 	      optcap_ptr += (cap_len + 2);
 	      optcap_len -= (cap_len + 2);
@@ -876,6 +900,9 @@ int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, struct bgp_attr
     case BGP_ATTR_PREFIX_SID:
       ret = bgp_attr_parse_prefix_sid(peer, attr_len, attr_extra, ptr, flag);
       break;
+    case BGP_ATTR_OTC:
+      ret = bgp_attr_parse_otc(peer, attr_len, attr_extra, ptr, flag);
+      break;
     default:
       ret = 0;
       break;
@@ -1018,8 +1045,8 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
   mpnhoplen = *ptr; ptr++;
   mpreachlen -= 4; /* 2+1+1 above */ 
   
-  /* IPv4 (4), RD+IPv4 (12), IPv6 (16), RD+IPv6 (24), IPv6 link-local+IPv6 global (32) */
-  if (mpnhoplen == 4 || mpnhoplen == 12 || mpnhoplen == 16 || mpnhoplen == 24 || mpnhoplen == 32) {
+  /* IPv4 (4), RD+IPv4 (12), IPv6 (16), RD+IPv6 (24), IPv6 link-local+IPv6 global (32), RD+IPv6+RD+IPv6 link-local (48) */
+  if (mpnhoplen == 4 || mpnhoplen == 12 || mpnhoplen == 16 || mpnhoplen == 24 || mpnhoplen == 32 || mpnhoplen == 48) {
     if (mpreachlen > mpnhoplen) {
       memset(&attr->mp_nexthop, 0, sizeof(struct host_addr));
 
@@ -1351,6 +1378,21 @@ int bgp_attr_parse_prefix_sid(struct bgp_peer *peer, u_int16_t len, struct bgp_a
   return SUCCESS;
 }
 
+/* OTC attribute. */
+int bgp_attr_parse_otc(struct bgp_peer *peer, u_int16_t len, struct bgp_attr_extra *attr_extra, char *ptr, u_char flag)
+{
+  u_int32_t tmp32;
+
+  /* Length check. */
+  if (len < 4) return ERR;
+
+  memcpy(&tmp32, ptr, 4);
+  attr_extra->otc = ntohl(tmp32);
+  ptr += len;
+
+  return SUCCESS;
+}
+
 int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, struct bgp_attr_extra *attr_extra, afi_t afi, safi_t safi, int idx)
 {
   struct bgp_peer *peer = bmd->peer;
@@ -1369,7 +1411,7 @@ int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, s
   if (!inter_domain_routing_db || !bms) return ERR;
 
   if (!bms->skip_rib) { 
-    modulo = bms->route_info_modulo(peer, &attr_extra->path_id, bms->table_per_peer_buckets);
+    modulo = bms->route_info_modulo(peer, &attr_extra->rd, &attr_extra->path_id, &bmd->extra, bms->table_per_peer_buckets);
     route = bgp_node_get(peer, inter_domain_routing_db->rib[afi][safi], p);
 
     /* Check previously received route. */
@@ -1497,7 +1539,7 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
   if (!inter_domain_routing_db || !bms) return ERR;
 
   if (!bms->skip_rib) {
-    modulo = bms->route_info_modulo(peer, &attr_extra->path_id, bms->table_per_peer_buckets);
+    modulo = bms->route_info_modulo(peer, &attr_extra->rd, &attr_extra->path_id, &bmd->extra, bms->table_per_peer_buckets);
 
     /* Lookup node. */
     route = bgp_node_get(peer, inter_domain_routing_db->rib[afi][safi], p);
