@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -38,7 +38,7 @@ int bgp_afi2family (int afi)
     return AF_INET;
   else if (afi == AFI_IP6)
     return AF_INET6;
-  return SUCCESS;
+  return AF_UNSPEC;
 }
 
 u_int16_t bgp_rd_type_get(u_int16_t type)
@@ -74,6 +74,8 @@ const char *bgp_rd_origin_print(u_int16_t type)
     return bgp_rd_origin[2];
   case RD_ORIGIN_FLOW:
     return bgp_rd_origin[3];
+  case RD_ORIGIN_MAP:
+    return bgp_rd_origin[4];
   default:
     return bgp_rd_origin[0];
   };
@@ -342,6 +344,17 @@ struct bgp_attr_extra *bgp_attr_extra_process(struct bgp_peer *peer, struct bgp_
     }
   }
 
+  /* OTC: either > 0 or absent */
+  if (attr_extra->otc) {
+    if (!rie) {
+      rie = bgp_attr_extra_get(ri);
+    }
+
+    if (rie) {
+      rie->otc = attr_extra->otc;
+    }
+  }
+
   if (rie && !(attr_extra->bitmap & BGP_BMAP_ATTR_AIGP)) rie->bitmap &= ~BGP_BMAP_ATTR_AIGP;
 
   return rie;
@@ -458,7 +471,7 @@ int attrhash_cmp(const void *p1, const void *p2)
   const struct bgp_attr *attr1 = (const struct bgp_attr *)p1;
   const struct bgp_attr *attr2 = (const struct bgp_attr *)p2;
 
-  if (attr1->flag == attr2->flag
+  if (attr1->rpki_maxlen == attr2->rpki_maxlen
       && attr1->bitmap == attr2->bitmap
       && attr1->origin == attr2->origin
       && attr1->nexthop.s_addr == attr2->nexthop.s_addr
@@ -669,7 +682,7 @@ struct bgp_peer *bgp_peer_cache_search(struct bgp_peer_cache_bucket *cache, u_in
   return ret;
 }
 
-int bgp_peer_init(struct bgp_peer *peer, int type)
+int bgp_peer_init(struct bgp_peer *peer, int type, int buflen)
 {
   struct bgp_misc_structs *bms;
   int ret = TRUE;
@@ -681,7 +694,7 @@ int bgp_peer_init(struct bgp_peer *peer, int type)
   memset(peer, 0, sizeof(struct bgp_peer));
   peer->type = type;
   peer->status = Idle;
-  peer->buf.tot_len = BGP_BUFFER_SIZE;
+  peer->buf.tot_len = buflen;
   peer->buf.base = malloc(peer->buf.tot_len);
   if (!peer->buf.base) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_peer_init). Exiting ..\n", config.name, bms->log_str);
@@ -693,7 +706,7 @@ int bgp_peer_init(struct bgp_peer *peer, int type)
   }
 
   if (config.bgp_xconnect_map) {
-    peer->xbuf.tot_len = BGP_BUFFER_SIZE;
+    peer->xbuf.tot_len = buflen;
     peer->xbuf.base = malloc(peer->xbuf.tot_len);
     if (!peer->xbuf.base) {
       Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_peer_init). Exiting ..\n", config.name, bms->log_str);
@@ -899,7 +912,7 @@ void bgp_table_info_delete(struct bgp_peer *peer, struct bgp_table *table, afi_t
     struct bgp_info *ri;
     struct bgp_info *ri_next;
 
-    if (bms->route_info_modulo) modulo = bms->route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
+    if (bms->route_info_modulo) modulo = bms->route_info_modulo(peer, NULL, NULL, NULL, bms->table_per_peer_buckets);
     else modulo = 0;
 
     for (peer_buckets = 0; peer_buckets < bms->table_per_peer_buckets; peer_buckets++) {
