@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -35,6 +35,9 @@
 #ifdef WITH_AVRO
 #include "plugin_cmn_avro.h"
 #endif
+#ifdef WITH_REDIS
+#include "ha.h"
+#endif
 
 int bmp_log_msg(struct bgp_peer *peer, struct bmp_data *bdata, struct pm_list *tlvs, bgp_tag_t *tag,
 		void *log_data, u_int64_t log_seq, char *event_type, int output, int log_type)
@@ -327,7 +330,15 @@ int bmp_log_msg(struct bgp_peer *peer, struct bmp_data *bdata, struct pm_list *t
       }
 
       if (output == PRINT_OUTPUT_AVRO_BIN) {
-	kafka_ret = write_binary_kafka(peer->log->kafka_host, p_avro_local_buf, p_avro_len);
+        if (bmp_bgp_forwarding) {
+          kafka_ret = write_binary_kafka(peer->log->kafka_host, p_avro_local_buf, p_avro_len);
+        }
+#ifdef WITH_REDIS
+        else {
+          /*BMP-BGP-HA stand-by - intercept BMP packet*/
+          bmp_bgp_ha_enqueue(p_avro_local_buf, p_avro_len);
+        }
+#endif
       }
       else if (output == PRINT_OUTPUT_AVRO_JSON) {
 	char *avro_local_str = NULL;
@@ -1090,6 +1101,18 @@ int bmp_log_msg_peer_down(struct bgp_peer *peer, struct bmp_data *bdata, struct
       json_object_set_new_nocheck(obj, "peer_type_str", json_string(bmp_peer_types[bdata->chars.peer_type]));
     }
 
+    if (!bdata->chars.is_loc && !bdata->chars.is_out) {
+      json_object_set_new_nocheck(obj, "is_post", json_integer((json_int_t)bdata->chars.is_post));
+      json_object_set_new_nocheck(obj, "is_in", json_integer(1));
+    }
+    else if (bdata->chars.is_loc) {
+      json_object_set_new_nocheck(obj, "is_loc", json_integer((json_int_t)bdata->chars.is_loc));
+    }
+    else if (bdata->chars.is_out) {
+      json_object_set_new_nocheck(obj, "is_post", json_integer((json_int_t)bdata->chars.is_post));
+      json_object_set_new_nocheck(obj, "is_out", json_integer((json_int_t)bdata->chars.is_out));
+    }
+
     if (!is_empty_256b(&bdata->chars.rd, sizeof(bdata->chars.rd))) {
       char rd_str[SHORTSHORTBUFLEN];
 
@@ -1172,6 +1195,51 @@ int bmp_log_msg_peer_down(struct bgp_peer *peer, struct bmp_data *bdata, struct
       pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
     }
 
+    if (!bdata->chars.is_loc && !bdata->chars.is_out) {
+      pm_avro_check(avro_value_get_by_name(obj, "is_in", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, TRUE, &p_avro_branch));
+      pm_avro_check(avro_value_set_int(&p_avro_branch, 1));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_post", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, TRUE, &p_avro_branch));
+      pm_avro_check(avro_value_set_int(&p_avro_branch, bdata->chars.is_post));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_loc", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_out", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+    }
+    else if (bdata->chars.is_loc) {
+      pm_avro_check(avro_value_get_by_name(obj, "is_in", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_loc", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, TRUE, &p_avro_branch));
+      pm_avro_check(avro_value_set_int(&p_avro_branch, bdata->chars.is_loc));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_post", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_out", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+    }
+    else if (bdata->chars.is_out) {
+      pm_avro_check(avro_value_get_by_name(obj, "is_in", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_loc", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, FALSE, &p_avro_branch));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_post", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, TRUE, &p_avro_branch));
+      pm_avro_check(avro_value_set_int(&p_avro_branch, bdata->chars.is_post));
+
+      pm_avro_check(avro_value_get_by_name(obj, "is_out", &p_avro_field, NULL));
+      pm_avro_check(avro_value_set_branch(&p_avro_field, TRUE, &p_avro_branch));
+      pm_avro_check(avro_value_set_int(&p_avro_branch, bdata->chars.is_out));
+    }
+
     if (!is_empty_256b(&bdata->chars.rd, sizeof(bdata->chars.rd))) {
       char rd_str[SHORTSHORTBUFLEN];
 
@@ -1598,9 +1666,12 @@ void bmp_handle_dump_event(int max_peers_idx)
     bmp_dump_workers_pool = allocate_thread_pool(config.bmp_dump_workers);
     assert(bmp_dump_workers_pool);
 
-    for (idx = 0; idx < config.bmp_dump_workers; idx++) {
-      if (!pdr[idx].noop) {
-        send_to_pool(bmp_dump_workers_pool, bmp_dump_event_runner, &pdr[idx]);
+    /* (BMP-BGP-HA feature) don't perform the dump in case the daemon is stand-by */
+    if (bmp_bgp_forwarding) {
+      for (idx = 0; idx < config.bmp_dump_workers; idx++) {
+        if (!pdr[idx].noop) {
+          send_to_pool(bmp_dump_workers_pool, bmp_dump_event_runner, &pdr[idx]);
+        }
       }
     }
 
@@ -1844,7 +1915,7 @@ int bmp_dump_event_runner(struct pm_dump_runner *pdr)
           node = bgp_table_top(peer, table);
 
           while (node) {
-            u_int32_t modulo = bms->route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
+            u_int32_t modulo = bms->route_info_modulo(peer, NULL, NULL, NULL, bms->table_per_peer_buckets);
             u_int32_t peer_buckets;
             struct bgp_info *ri;
 
@@ -2218,6 +2289,11 @@ avro_schema_t p_avro_schema_build_bmp_peer_down(char *schema_name)
   avro_schema_record_field_append(schema, "peer_type", avro_schema_int());
   avro_schema_record_field_append(schema, "peer_type_str", optstr_s);
 
+  avro_schema_record_field_append(schema, "is_in", optint_s);
+  avro_schema_record_field_append(schema, "is_loc", optint_s);
+  avro_schema_record_field_append(schema, "is_post", optint_s);
+  avro_schema_record_field_append(schema, "is_out", optint_s);
+
   avro_schema_record_field_append(schema, "rd", optstr_s);
   avro_schema_record_field_append(schema, "rd_origin", optstr_s);
 
@@ -2355,6 +2431,7 @@ void p_avro_schema_build_bmp_common(avro_schema_t *schema, avro_schema_t *optlon
   avro_schema_record_field_append((*schema), "bmp_msg_type", avro_schema_string());
   avro_schema_record_field_append((*schema), "writer_id", avro_schema_string());
 
+
   if (config.bmp_daemon_tag_map) {
     avro_schema_record_field_append((*schema), "tag", (*optlong_s));
 
