@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -24,9 +24,9 @@
 #include "bgp/bgp.h"
 #include "bmp.h"
 
-void bmp_srcdst_lookup(struct packet_ptrs *pptrs)
+void bmp_srcdst_lookup(struct packet_ptrs *pptrs, struct bgp_lookup_info *bl_info)
 {
-  bgp_srcdst_lookup(pptrs, FUNC_TYPE_BMP);
+  bgp_srcdst_lookup(pptrs, FUNC_TYPE_BMP, bl_info);
 }
 
 struct bgp_peer *bgp_lookup_find_bmp_peer(struct sockaddr *sa, struct xflow_status_entry *xs_entry, u_int16_t l3_proto, int compare_bgp_port)
@@ -86,7 +86,7 @@ struct bgp_peer *bgp_lookup_find_bmp_peer(struct sockaddr *sa, struct xflow_stat
   return peer;
 }
 
-u_int32_t bmp_route_info_modulo_pathid(struct bgp_peer *peer, path_id_t *path_id, int per_peer_buckets)
+u_int32_t bmp_route_info_modulo_pathid(struct bgp_peer *peer, rd_t *rd, path_id_t *path_id, struct bgp_msg_extra_data *bmed, int per_peer_buckets)
 {
   struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   struct bmp_peer *bmpp = peer->bmp_se;
@@ -105,6 +105,31 @@ u_int32_t bmp_route_info_modulo_pathid(struct bgp_peer *peer, path_id_t *path_id
           (bms->table_peer_buckets * per_peer_buckets));
 }
 
+u_int32_t bmp_route_info_modulo_mplsvpnrd(struct bgp_peer *peer, rd_t *rd, path_id_t *path_id, struct bgp_msg_extra_data *bmed, int per_peer_buckets)
+{
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
+  struct bmp_peer *bmpp = peer->bmp_se;
+  u_int16_t local_rd = 0;
+  int fd = 0;
+
+  if (bmed) { /* BMP message being parsed (RD is stored in bmed->data->rd) */
+    struct bmp_chars *bmed_bmp_chars = (struct bmp_chars *) bmed->data;
+    if (bmed_bmp_chars->rd.val) local_rd = (bmed_bmp_chars->rd.type + bmed_bmp_chars->rd.as + bmed_bmp_chars->rd.val);
+  }
+  else { /* Correlating with flow */
+    if (rd) local_rd = (rd->type + rd->as + rd->val);
+  }
+
+  if (peer->fd) fd = peer->fd;
+  else {
+    if (bmpp && bmpp->self.fd) fd = bmpp->self.fd;
+  }
+
+  return (((fd * per_peer_buckets) +
+          (local_rd % per_peer_buckets)) %
+          (bms->table_peer_buckets * per_peer_buckets));
+}
+
 int bgp_lookup_node_match_cmp_bmp(struct bgp_info *info, struct node_match_cmp_term2 *nmct2)
 {
   struct bmp_peer *bmpp = info->peer->bmp_se;
