@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -26,9 +26,9 @@
 
 /* defines */
 #define CFG_LINE_LEN(x) (SRVBUFLEN-strlen(x))
-#define MAX_CUSTOM_PRIMITIVES		64
-#define MAX_CUSTOM_PRIMITIVE_NAMELEN	64
-#define MAX_CUSTOM_PRIMITIVE_PD_PTRS	8
+#define MAX_CUSTOM_PRIMITIVES           64
+#define MAX_CUSTOM_PRIMITIVE_NAMELEN    64
+#define MAX_CUSTOM_PRIMITIVE_PD_PTRS    8
 
 /* structures */
 struct _dictionary_line {
@@ -75,10 +75,12 @@ struct custom_primitives_ptrs {
 };
 
 struct configuration {
-  pm_cfgreg_t what_to_count;	/* first registry */
-  pm_cfgreg_t what_to_count_2;	/* second registry */
+  pm_cfgreg_t what_to_count;    /* first registry */
+  pm_cfgreg_t what_to_count_2;  /* second registry */
+  pm_cfgreg_t what_to_count_3;  /* third registry */
   pm_cfgreg_t nfprobe_what_to_count;
   pm_cfgreg_t nfprobe_what_to_count_2;
+  pm_cfgreg_t nfprobe_what_to_count_3;
   char *aggregate_primitives;
   int aggregate_unknown_etype;
   struct custom_primitives_ptrs cpptrs;
@@ -97,8 +99,8 @@ struct configuration {
   int redis_db;
   int sock;
   int bgp_sock;
-  int acct_type; 
-  int data_type; 
+  int acct_type;
+  int data_type;
   int pipe_homegrown;
   u_int64_t pipe_size;
   u_int64_t buffer_size;
@@ -126,7 +128,6 @@ struct configuration {
   pm_dtls_glob_t dtls_globs;
 #endif
   char *imt_plugin_path;
-  char *imt_plugin_passwd;
   char *sql_db;
   char *sql_table;
   char *sql_table_schema;
@@ -246,8 +247,10 @@ struct configuration {
   int sfacctd_counter_kafka_broker_port;
   int sfacctd_counter_kafka_retry;
   char *sfacctd_counter_kafka_config_file;
-  int nfacctd_disable_checks;
+  int nfacctd_disable_sanity_checks;
   int nfacctd_disable_opt_scope_check;
+  int nfacctd_ignore_exporter_address;
+  int nfacctd_pre_processing_checks;
   int telemetry_daemon;
   int telemetry_sock;
   int telemetry_port_tcp;
@@ -260,12 +263,9 @@ struct configuration {
   int telemetry_udp_notif_ipv6_only;
   int telemetry_udp_notif_nmsgs;
   char *telemetry_udp_notif_rp_ebpf_prog;
+  char *telemetry_grpc_collector_socket;
+  char *telemetry_grpc_collector_conf;
   int telemetry_ipv6_only;
-  char *telemetry_zmq_address;
-  char *telemetry_kafka_broker_host;
-  int telemetry_kafka_broker_port;
-  char *telemetry_kafka_topic;
-  char *telemetry_kafka_config_file;
   char *telemetry_decoder;
   int telemetry_decoder_id;
   int telemetry_max_peers;
@@ -478,6 +478,7 @@ struct configuration {
   char *bmp_dump_avro_schema_file;
   int bmp_dump_refresh_time;
   int bmp_dump_time_slots;
+  int bmp_dump_exclude_stats;
   char *bmp_dump_amqp_host;
   char *bmp_dump_amqp_vhost;
   char *bmp_dump_amqp_user;
@@ -508,10 +509,6 @@ struct configuration {
   char *igp_daemon_map_msglog;
   char *geoip_ipv4_file;
   char *geoip_ipv6_file;
-#if defined WITH_GEOIP
-  GeoIP *geoip_ipv4;
-  GeoIP *geoip_ipv6;
-#endif
   char *geoipv2_file;
 #if defined WITH_GEOIPV2
   MMDB_s geoipv2_db;
@@ -533,9 +530,9 @@ struct configuration {
   int buckets;
   int daemon;
   int active_plugins;
-  char *logfile; 
-  FILE *logfile_fd; 
-  char *pidfile; 
+  char *logfile;
+  FILE *logfile_fd;
+  char *pidfile;
   int networks_mask;
   char *networks_file;
   int networks_file_filter;
@@ -553,6 +550,8 @@ struct configuration {
   struct pretag_label_filter ptlf;
   int pretag_label_encode_as_map;
   int tcpflags_encode_as_array;
+  int bgp_comms_encode_as_array;
+  int as_path_encode_as_array;
   int mpls_label_stack_encode_as_array;
   int fwd_status_encode_as_string;
   int tos_encode_as_dscp;
@@ -561,6 +560,7 @@ struct configuration {
   int maps_entries;
   int maps_row_len;
   char *pre_tag_map;
+  int pre_tag_map_dont_recirculate;
   struct id_table ptm;
   int ptm_alloc;
   int ptm_global;
@@ -573,10 +573,10 @@ struct configuration {
   char *syslog;
   int debug;
   int debug_internal_msg;
+  int dry_run;
   int snaplen;
   int classifier_table_num;
   int classifier_ndpi;
-  u_int32_t ndpi_num_roots;
   u_int32_t ndpi_max_flows;
   int ndpi_proto_guess;
   u_int32_t ndpi_idle_scan_period;
@@ -623,12 +623,12 @@ struct configuration {
   int pcap_arista_trailer_flag_value;
   int dump_max_writers;
   int tmp_asa_bi_flow;
-  int tmp_vlan_legacy;
   int tmp_bgp_lookup_compare_ports;
   int tmp_bgp_daemon_route_refresh;
   int tmp_bgp_daemon_origin_type_int;
   int tmp_telemetry_udp_notif_legacy;
   int tmp_telemetry_decode_cisco_v1_json_string;
+  int bgp_bmp_daemon_ha;
   size_t thread_stack;
   char *rpki_roas_file;
   char *rpki_rtr_cache;
@@ -638,7 +638,7 @@ struct configuration {
   int bmp_daemon_parse_proxy_header;
 };
 
-/* prototypes */ 
+/* prototypes */
 extern void evaluate_configuration(char *, int);
 extern int parse_configuration_file(char *);
 extern int parse_plugin_names(char *, int, int);
