@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -30,6 +30,7 @@ extern void cfg_key_legacy_warning(char *, char *);
 
 extern int cfg_key_debug(char *, char *, char *);
 extern int cfg_key_debug_internal_msg(char *, char *, char *);
+extern int cfg_key_dry_run(char *, char *, char *);
 extern int cfg_key_syslog(char *, char *, char *);
 extern int cfg_key_logfile(char *, char *, char *);
 extern int cfg_key_pidfile(char *, char *, char *);
@@ -70,7 +71,6 @@ extern int cfg_key_promisc(char *, char *, char *);
 extern int cfg_key_num_protos(char *, char *, char *);
 extern int cfg_key_num_hosts(char *, char *, char *);
 extern int cfg_key_imt_path(char *, char *, char *);
-extern int cfg_key_imt_passwd(char *, char *, char *);
 extern int cfg_key_imt_buckets(char *, char *, char *);
 extern int cfg_key_imt_mem_pools_number(char *, char *, char *);
 extern int cfg_key_imt_mem_pools_size(char *, char *, char *);
@@ -163,8 +163,9 @@ extern int cfg_key_nfacctd_time_secs(char *, char *, char *);
 extern int cfg_key_nfacctd_time_new(char *, char *, char *);
 extern int cfg_key_nfacctd_as_new(char *, char *, char *);
 extern int cfg_key_nfacctd_net(char *, char *, char *);
-extern int cfg_key_nfacctd_disable_checks(char *, char *, char *);
+extern int cfg_key_nfacctd_disable_sanity_checks(char *, char *, char *);
 extern int cfg_key_nfacctd_disable_opt_scope_check(char *, char *, char *);
+extern int cfg_key_nfacctd_ignore_exporter_address(char *, char *, char *);
 extern int cfg_key_nfacctd_mcast_groups(char *, char *, char *);
 extern int cfg_key_nfacctd_pipe_size(char *, char *, char *);
 extern int cfg_key_nfacctd_pro_rating(char *, char *, char *);
@@ -221,12 +222,8 @@ extern int cfg_key_telemetry_udp_notif_interface(char *, char *, char *);
 extern int cfg_key_telemetry_udp_notif_ipv6_only(char *, char *, char *);
 extern int cfg_key_telemetry_udp_notif_nmsgs(char *, char *, char *);
 extern int cfg_key_telemetry_udp_notif_rp_ebpf_prog(char *, char *, char *);
+extern int cfg_key_telemetry_grpc_collector_conf(char *, char *, char *);
 extern int cfg_key_telemetry_ipv6_only(char *, char *, char *);
-extern int cfg_key_telemetry_zmq_address(char *, char *, char *);
-extern int cfg_key_telemetry_kafka_broker_host(char *, char *, char *);
-extern int cfg_key_telemetry_kafka_broker_port(char *, char *, char *);
-extern int cfg_key_telemetry_kafka_topic(char *, char *, char *);
-extern int cfg_key_telemetry_kafka_config_file(char *, char *, char *);
 extern int cfg_key_telemetry_decoder(char *, char *, char *);
 extern int cfg_key_telemetry_max_peers(char *, char *, char *);
 extern int cfg_key_telemetry_peer_timeout(char *, char *, char *);
@@ -285,11 +282,14 @@ extern int cfg_key_maps_index(char *, char *, char *);
 extern int cfg_key_maps_entries(char *, char *, char *);
 extern int cfg_key_maps_row_len(char *, char *, char *);
 extern int cfg_key_pre_tag_map(char *, char *, char *);
+extern int cfg_key_pre_tag_map_dont_recirculate(char *, char *, char *);
 extern int cfg_key_pre_tag_filter(char *, char *, char *);
 extern int cfg_key_pre_tag2_filter(char *, char *, char *);
 extern int cfg_key_pre_tag_label_filter(char *, char *, char *);
 extern int cfg_key_pre_tag_label_encode_as_map(char *, char *, char *);
 extern int cfg_key_tcpflags_encode_as_array(char *, char *, char *);
+extern int cfg_key_bgp_comms_encode_as_array(char *, char *, char *);
+extern int cfg_key_as_path_encode_as_array(char *, char *, char *);
 extern int cfg_key_fwd_status_encode_as_string(char *, char *, char *);
 extern int cfg_key_tos_encode_as_dscp(char *, char *, char *);
 extern int cfg_key_post_tag(char *, char *, char *);
@@ -482,6 +482,7 @@ extern int cfg_key_bmp_daemon_dump_avro_schema_file(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_latest_file(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_refresh_time(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_time_slots(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_exclude_stats(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_amqp_host(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_amqp_vhost(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_amqp_user(char *, char *, char *);
@@ -530,6 +531,8 @@ extern int cfg_key_tmp_bgp_daemon_route_refresh(char *, char *, char *);
 extern int cfg_key_tmp_bgp_daemon_origin_type_int(char *, char *, char *);
 extern int cfg_key_tmp_telemetry_daemon_udp_notif_legacy(char *, char *, char *);
 extern int cfg_key_tmp_telemetry_decode_cisco_v1_json_string(char *, char *, char *);
+extern int cfg_key_bgp_bmp_daemon_ha(char *, char *, char *);
+extern int cfg_key_nfacctd_pre_processing_checks(char *, char *, char *);
 
 extern void parse_time(char *, char *, int *, int *);
 extern void cfg_get_primitive_index_value(u_int64_t, u_int64_t *, u_int64_t *);
