@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -70,17 +70,18 @@ struct query_header {
   int type;				/* type of query */
   pm_cfgreg_t what_to_count;		/* aggregation */
   pm_cfgreg_t what_to_count_2;		/* aggregation */
+  pm_cfgreg_t what_to_count_3;		/* aggregation */
   unsigned int num;			/* number of queries */
   unsigned int ip_sz;			/* IP addresses size (in bytes) */
   unsigned int cnt_sz;			/* counters size (in bytes) */
   struct extra_primitives extras;	/* offsets for non-standard aggregation primitives structures */
   int datasize;				/* total length of aggregation primitives structures */
-  char passwd[12];			/* OBSOLETED: password */
 };
 
 struct query_entry {
   pm_cfgreg_t what_to_count;			/* aggregation */
   pm_cfgreg_t what_to_count_2;			/* aggregation */
+  pm_cfgreg_t what_to_count_3;			/* aggregation */
   struct pkt_primitives data;			/* actual data */
   struct pkt_bgp_primitives pbgp;		/* extended BGP data */
   struct pkt_legacy_bgp_primitives plbgp;	/* extended BGP data */
@@ -139,7 +140,7 @@ extern int build_query_server(char *);
 extern void process_query_data(int, unsigned char *, int, struct extra_primitives *, int, int);
 extern void mask_elem(struct pkt_primitives *, struct pkt_bgp_primitives *, struct pkt_legacy_bgp_primitives *,
 			struct pkt_nat_primitives *, struct pkt_mpls_primitives *, struct pkt_tunnel_primitives *,
-			struct acc *, u_int64_t, u_int64_t, struct extra_primitives *);
+			struct acc *, u_int64_t, u_int64_t, u_int64_t, struct extra_primitives *);
 extern void enQueue_elem(int, struct reply_buffer *, void *, int, int);
 extern void Accumulate_Counters(struct pkt_data *, struct acc *);
 extern int test_zero_elem(struct acc *);
