@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -25,6 +25,7 @@
 #include "plugin_common.h"
 #include "kafka_common.h"
 #include "base64.h"
+#include "thread_pool.h"
 
 struct p_kafka_host kafkap_kafka_host;
 struct p_kafka_host bgp_daemon_msglog_kafka_host;
@@ -286,12 +287,14 @@ int p_kafka_parse_config_entry(char *buf, char *type, char **key, char **value)
 void p_kafka_apply_global_config(struct p_kafka_host *kafka_host)
 {
   FILE *file;
-  char buf[SRVBUFLEN], errstr[SRVBUFLEN], *key, *value;
+  char buf[SRVBUFLEN], errstr[LONGLONGSRVBUFLEN], *key, *value;
   int lineno = 1, ret;
 
   if (kafka_host && kafka_host->config_file && kafka_host->cfg) {
     if ((file = fopen(kafka_host->config_file, "r")) == NULL) {
-      Log(LOG_WARNING, "WARN ( %s/%s ): [%s] file not found. librdkafka global config not loaded.\n", config.name, config.type, kafka_host->config_file);
+      strerror_r(errno, errstr, sizeof(errstr));
+      Log(LOG_WARNING, "WARN ( %s/%s ): [%s] %s. librdkafka global config not loaded.\n",
+          config.name, config.type, kafka_host->config_file, errstr);
       return;
     }
     else Log(LOG_INFO, "INFO ( %s/%s ): [%s] Reading librdkafka global config.\n", config.name, config.type, kafka_host->config_file);
@@ -322,12 +325,14 @@ void p_kafka_apply_global_config(struct p_kafka_host *kafka_host)
 void p_kafka_apply_topic_config(struct p_kafka_host *kafka_host)
 {
   FILE *file;
-  char buf[SRVBUFLEN], errstr[SRVBUFLEN], *key, *value;
+  char buf[SRVBUFLEN], errstr[LONGLONGSRVBUFLEN], *key, *value;
   int lineno = 1, ret;
 
   if (kafka_host && kafka_host->config_file && kafka_host->topic_cfg) {
     if ((file = fopen(kafka_host->config_file, "r")) == NULL) {
-      Log(LOG_WARNING, "WARN ( %s/%s ): [%s] file not found. librdkafka topic configuration not loaded.\n", config.name, config.type, kafka_host->config_file);
+      strerror_r(errno, errstr, sizeof(errstr));
+      Log(LOG_WARNING, "WARN ( %s/%s ): [%s] %s. librdkafka topic configuration not loaded.\n",
+          config.name, config.type, kafka_host->config_file, errstr);
       return;
     }
     else Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] Reading librdkafka topic configuration.\n", config.name, config.type, kafka_host->config_file);
@@ -434,7 +439,7 @@ int p_kafka_produce_data_to_part(struct p_kafka_host *kafka_host, void *data, si
   int flag = RD_KAFKA_MSG_F_COPY;
 
   kafkap_ret_err_cb = FALSE;
-
+  
   if (do_free) {
     flag = RD_KAFKA_MSG_F_FREE;
   }
@@ -450,10 +455,12 @@ int p_kafka_produce_data_to_part(struct p_kafka_host *kafka_host, void *data, si
       return ERR;
     }
   }
-  else return ERR;
+
+  else {
+    return ERR;
+  }
 
   rd_kafka_poll(kafka_host->rk, 0);
-
   return ret; 
 }
 
