@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -161,6 +161,12 @@ void mongodb_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   }
 #endif
 
+  if (config.dry_run == DRY_RUN_SETUP) {
+    sleep(DEFAULT_SLOTH_SLEEP_TIME); /* Make sure all comes up */
+    printf("INFO ( %s/%s ): Dry run 'setup'. Exiting ..\n", config.name, config.type);
+    exit(0);
+  }
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -514,12 +520,11 @@ void MongoDB_cache_purge(struct chained_cache *queue[], int index, int safe_acti
       }
   
       if (config.what_to_count & COUNT_VLAN) {
-	if (config.tmp_vlan_legacy) {
-	  bson_append_int(bson_elem, "vlan", data->vlan_id);
-	}
-	else {
-	  bson_append_int(bson_elem, "vlan_in", data->vlan_id);
-	}
+	bson_append_int(bson_elem, "vlan", data->vlan_id);
+      }
+
+      if (config.what_to_count_3 & COUNT_IN_VLAN) {
+	bson_append_int(bson_elem, "vlan_in", data->vlan_id);
       }
 
       if (config.what_to_count_2 & COUNT_OUT_VLAN) {
@@ -694,21 +699,7 @@ void MongoDB_cache_purge(struct chained_cache *queue[], int index, int safe_acti
       if (config.what_to_count & (COUNT_SRC_PORT|COUNT_SUM_PORT)) bson_append_int(bson_elem, "port_src", data->src_port);
       if (config.what_to_count & COUNT_DST_PORT) bson_append_int(bson_elem, "port_dst", data->dst_port);
 
-  #if defined (WITH_GEOIP)
-      if (config.what_to_count_2 & COUNT_SRC_HOST_COUNTRY) {
-        if (data->src_ip_country.id > 0)
-  	  bson_append_string(bson_elem, "country_ip_src", GeoIP_code_by_id(data->src_ip_country.id));
-        else
-  	  bson_append_null(bson_elem, "country_ip_src");
-      }
-      if (config.what_to_count_2 & COUNT_DST_HOST_COUNTRY) {
-        if (data->dst_ip_country.id > 0)
-  	  bson_append_string(bson_elem, "country_ip_dst", GeoIP_code_by_id(data->dst_ip_country.id));
-        else
-  	  bson_append_null(bson_elem, "country_ip_dst");
-      }
-  #endif
-  #if defined (WITH_GEOIPV2)
+  #if defined WITH_GEOIPV2
       if (config.what_to_count_2 & COUNT_SRC_HOST_COUNTRY) {
         if (strlen(data->src_ip_country.str))
           bson_append_string(bson_elem, "country_ip_src", data->src_ip_country.str);
@@ -757,6 +748,7 @@ void MongoDB_cache_purge(struct chained_cache *queue[], int index, int safe_acti
       }
   
       if (config.what_to_count & COUNT_IP_TOS) bson_append_int(bson_elem, "tos", data->tos);
+      if (config.what_to_count_3 & COUNT_FLOW_LABEL) bson_append_int(bson_elem, "flow_label", data->flow_label);
       if (config.what_to_count_2 & COUNT_SAMPLING_RATE) bson_append_int(bson_elem, "sampling_rate", data->sampling_rate);
       if (config.what_to_count_2 & COUNT_SAMPLING_DIRECTION) bson_append_string(bson_elem, "sampling_direction",
 										sampling_direction_print(data->sampling_direction));
@@ -792,6 +784,10 @@ void MongoDB_cache_purge(struct chained_cache *queue[], int index, int safe_acti
 	bson_append_int(bson_elem, "mpls_label_stack", label_stack);
       }
 
+      if (config.what_to_count_2 & COUNT_PATH_DELAY_AVG_USEC) bson_append_int(bson_elem, "path_delay_avg_usec", pmpls->path_delay_avg_usec);
+      if (config.what_to_count_2 & COUNT_PATH_DELAY_MIN_USEC) bson_append_int(bson_elem, "path_delay_min_usec", pmpls->path_delay_min_usec);
+      if (config.what_to_count_2 & COUNT_PATH_DELAY_MAX_USEC) bson_append_int(bson_elem, "path_delay_max_usec", pmpls->path_delay_max_usec);
+
       if (config.what_to_count_2 & COUNT_TUNNEL_SRC_MAC) {
         etheraddr_string(ptun->tunnel_eth_shost, src_mac);
         bson_append_string(bson_elem, "tunnel_mac_src", src_mac);
@@ -815,10 +811,12 @@ void MongoDB_cache_purge(struct chained_cache *queue[], int index, int safe_acti
       }
 
       if (config.what_to_count_2 & COUNT_TUNNEL_IP_TOS) bson_append_int(bson_elem, "tunnel_tos", ptun->tunnel_tos);
+      if (config.what_to_count_3 & COUNT_TUNNEL_FLOW_LABEL) bson_append_int(bson_elem, "tunnel_flow_label", ptun->tunnel_flow_label);
       if (config.what_to_count_2 & COUNT_TUNNEL_SRC_PORT) bson_append_int(bson_elem, "tunnel_port_src", ptun->tunnel_src_port);
       if (config.what_to_count_2 & COUNT_TUNNEL_DST_PORT) bson_append_int(bson_elem, "tunnel_port_dst", ptun->tunnel_dst_port);
       if (config.what_to_count_2 & COUNT_TUNNEL_TCPFLAGS) bson_append_int(bson_elem, "tunnel_tcp_flags", data->tunnel_tcp_flags);
       if (config.what_to_count_2 & COUNT_VXLAN) bson_append_int(bson_elem, "vxlan", ptun->tunnel_id);
+      if (config.what_to_count_3 & COUNT_NVGRE) bson_append_int(bson_elem, "nvgre", ptun->nvgre_tunnel_id);
   
       if (config.what_to_count_2 & COUNT_TIMESTAMP_START) {
 	if (config.timestamps_since_epoch) {
