@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -41,6 +41,7 @@ char mysql_table_v5[] = "acct_v5";
 char mysql_table_v6[] = "acct_v6";
 char mysql_table_v7[] = "acct_v7";
 char mysql_table_v8[] = "acct_v8";
+char mysql_table_v9[] = "acct_v9";
 char mysql_table_bgp[] = "acct_bgp";
 
 /* Functions */
@@ -133,6 +134,12 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   }
 #endif
 
+  if (config.dry_run == DRY_RUN_SETUP) {
+    sleep(DEFAULT_SLOTH_SLEEP_TIME); /* Make sure all comes up */
+    printf("INFO ( %s/%s ): Dry run 'setup'. Exiting ..\n", config.name, config.type);
+    exit(0);
+  }
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -659,6 +666,9 @@ int MY_compose_static_queries()
     }
   }
 
+  /* SET time_zone */
+  strncpy(timezone_clause, "SET time_zone = '+00:00'", sizeof(timezone_clause));
+
   return primitives;
 }
 
@@ -679,6 +689,17 @@ void MY_Unlock(struct BE_descs *bed)
   if (bed->b->connected) mysql_query(bed->b->desc, unlock_clause);
 }
 
+void MY_Set_Timezone(struct DBdesc *db)
+{
+  if (!db->fail) {
+    if (mysql_query(db->desc, timezone_clause)) {
+      MY_get_errmsg(db);
+      sql_db_warnmsg(db);
+      /* non fatal error */
+    }
+  }
+}
+
 void MY_DB_Connect(struct DBdesc *db, char *host)
 {
   bool reconnect = TRUE;
@@ -712,6 +733,10 @@ void MY_DB_Connect(struct DBdesc *db, char *host)
       sql_db_ok(db);
     }
   }
+
+  if (config.timestamps_utc) {
+    MY_Set_Timezone(db);
+  }
 }
 
 void MY_DB_Close(struct BE_descs *bed)
@@ -771,6 +796,7 @@ void MY_init_default_values(struct insert_data *idata)
   if (!config.sql_port) config.sql_port = mysql_prt;
   if (!config.sql_table) {
     if (config.sql_table_version == (SQL_TABLE_VERSION_BGP+1)) config.sql_table = mysql_table_bgp;
+    else if (config.sql_table_version == 9) config.sql_table = mysql_table_v9;
     else if (config.sql_table_version == 8) config.sql_table = mysql_table_v8;
     else if (config.sql_table_version == 7) config.sql_table = mysql_table_v7;
     else if (config.sql_table_version == 6) config.sql_table = mysql_table_v6;
