@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -74,6 +74,7 @@ struct chdlc_header {
 
 /* Ethernet protocol ID's */
 #define ETHERTYPE_IP		0x0800          /* IP */
+#define ETHERTYPE_ARP		0x0806          /* ARP */
 #define ETHERTYPE_IPV6          0x86dd		/* IPv6 */
 #define ETHERTYPE_PPPOE         0x8864          /* pppoe (session stage) */
 #define ETHERTYPE_8021Q		0x8100          /* 802.1Q */
@@ -254,14 +255,15 @@ typedef u_int8_t safi_t;
 #define RD_TYPE_AS4     2
 
 /* RD: squat squat */
-#define RD_TYPE_VRFID	0x1000
-#define RD_TYPE_MASK	0x1FFF
+#define RD_TYPE_VRFID	0x0800
+#define RD_TYPE_MASK	0x0FFF
 
 #define RD_ORIGIN_UNKNOWN	0x0000
+#define RD_ORIGIN_MAP		0x1000
 #define RD_ORIGIN_FLOW		0x2000
 #define RD_ORIGIN_BGP		0x4000
 #define RD_ORIGIN_BMP		0x8000
-#define RD_ORIGIN_MASK		0xE000
+#define RD_ORIGIN_MASK		0xF000
 
 #define RD_ORIGIN_FUNC_TYPE_MAX 2
 
@@ -411,6 +413,7 @@ struct pkt_primitives {
   u_int16_t dst_port;
   u_int8_t tos;
   u_int8_t proto;
+  u_int32_t flow_label;
   u_int32_t ifindex_in;
   u_int32_t ifindex_out;
 #if defined (WITH_GEOIP) || defined (WITH_GEOIPV2)
@@ -511,6 +514,9 @@ struct pkt_stitching {
 #define MAX_MPLS_LABELS         10
 #define MAX_MPLS_LABEL_LEN      16
 
+/* SRV6 */
+#define MAX_SRV6_SEGMENT_IPV6_LIST_ENTRIES	16
+
 struct extra_primitives {
   u_int16_t off_pkt_bgp_primitives;
   u_int16_t off_pkt_lbgp_primitives;
@@ -581,6 +587,9 @@ struct pkt_nat_primitives {
 struct pkt_mpls_primitives {
   u_int32_t mpls_label_top;
   u_int32_t mpls_label_bottom;
+  u_int32_t path_delay_avg_usec;
+  u_int32_t path_delay_min_usec;
+  u_int32_t path_delay_max_usec;
 };
 
 struct pkt_tunnel_primitives {
@@ -589,11 +598,18 @@ struct pkt_tunnel_primitives {
   struct host_addr tunnel_src_ip;
   struct host_addr tunnel_dst_ip;
   u_int8_t tunnel_tos;
+  u_int32_t tunnel_flow_label;
   u_int8_t tunnel_proto;
   u_int16_t tunnel_src_port;
   u_int16_t tunnel_dst_port;
   u_int8_t tunnel_tcp_flags;
+
   u_int32_t tunnel_id; /* ie. VXLAN VNI */
+  u_int32_t nvgre_tunnel_id; /* ie. NVGRE TNI */
+#if defined (HAVE_L2)
+  u_int16_t cvlan_id;
+  u_int16_t out_cvlan_id;
+#endif
 };
 
 /* same as pkt_legacy_bgp_primitives but pointers in place of strings */
