@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -198,7 +198,7 @@ format_time(time_t t)
 static const char *
 format_flow(struct FLOW *flow)
 {
-	char addr1[64], addr2[64], stime[20], ftime[20];
+	char addr1[64], addr2[64], stime[32], ftime[32];
 	static char buf[1024];
 
 	inet_ntop(flow->af, &flow->addr[0], addr1, sizeof(addr1));
@@ -1557,6 +1557,12 @@ sort_version:
   }
 #endif
 
+  if (config.dry_run == DRY_RUN_SETUP) {
+    sleep(DEFAULT_SLOTH_SLEEP_TIME); /* Make sure all comes up */
+    printf("INFO ( %s/%s ): Dry run 'setup'. Exiting ..\n", config.name, config.type);
+    exit(0);
+  }
+
   for(;;) {
     status->wakeup = TRUE;
     poll_bypass = FALSE;
