@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -22,7 +22,6 @@
 /* includes */
 #include "pmacct.h"
 #include "pmacct-data.h"
-#include "pmacct-dlt.h"
 #include "pretag_handlers.h"
 #include "plugin_hooks.h"
 #include "pkt_handlers.h"
@@ -154,14 +153,14 @@ void pm_pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *bu
         }
         if (config.bgp_daemon) {
           BTA_find_id((struct id_table *)pptrs.bta_table, &pptrs, &pptrs.bta, &pptrs.bta2);
-          bgp_srcdst_lookup(&pptrs, FUNC_TYPE_BGP);
+          bgp_srcdst_lookup(&pptrs, FUNC_TYPE_BGP, NULL);
         }
         if (config.bgp_daemon_peer_as_src_map) PM_find_id((struct id_table *)pptrs.bpas_table, &pptrs, &pptrs.bpas, NULL);
         if (config.bgp_daemon_src_local_pref_map) PM_find_id((struct id_table *)pptrs.blp_table, &pptrs, &pptrs.blp, NULL);
         if (config.bgp_daemon_src_med_map) PM_find_id((struct id_table *)pptrs.bmed_table, &pptrs, &pptrs.bmed, NULL);
         if (config.bmp_daemon) {
           BTA_find_id((struct id_table *)pptrs.bta_table, &pptrs, &pptrs.bta, &pptrs.bta2);
-	  bmp_srcdst_lookup(&pptrs);
+	  bmp_srcdst_lookup(&pptrs, NULL);
 	}
 
 	set_index_pkt_ptrs(&pptrs);
@@ -309,9 +308,9 @@ int ip_handler(register struct packet_ptrs *pptrs)
         if (((struct pm_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_FIN) pptrs->tcp_flags |= TH_FIN;
         if (((struct pm_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_RST) pptrs->tcp_flags |= TH_RST;
         if (((struct pm_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_ACK && pptrs->tcp_flags) pptrs->tcp_flags |= TH_ACK;
-      }
 
-      ip_flow_handler(pptrs);
+	ip_flow_handler(pptrs);
+      }
     }
 
     /* XXX: optimize/short circuit here! */
@@ -492,9 +491,9 @@ int ip6_handler(register struct packet_ptrs *pptrs)
         if (((struct pm_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_FIN) pptrs->tcp_flags |= TH_FIN;
         if (((struct pm_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_RST) pptrs->tcp_flags |= TH_RST;
         if (((struct pm_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_ACK && pptrs->tcp_flags) pptrs->tcp_flags |= TH_ACK;
-      }
 
-      ip_flow6_handler(pptrs);
+	ip_flow6_handler(pptrs);
+      }
     }
 
     /* XXX: optimize/short circuit here! */
@@ -507,6 +506,8 @@ int ip6_handler(register struct packet_ptrs *pptrs)
   pptrs->icmp_code = FALSE;
 
   if (pptrs->l4_proto == IPPROTO_ICMPV6) {
+    pptrs->tlh_ptr = ptr;
+
     pptrs->icmp_type = ((struct pm_icmphdr *)pptrs->tlh_ptr)->type;
     pptrs->icmp_code = ((struct pm_icmphdr *)pptrs->tlh_ptr)->code;
   }
