@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -30,6 +30,7 @@ extern void evaluate_packet_handlers();
 extern void src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void cvlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void cos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -68,13 +69,14 @@ extern void timestamp_arrival_handler(struct channels_list_entry *, struct packe
 extern void custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void sfprobe_payload_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void nfprobe_extras_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void mpls_vpn_rd_frommap_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
 extern void NF_src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_in_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_out_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_cvlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_out_cvlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_cos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -91,6 +93,7 @@ extern void NF_peer_src_ip_handler(struct channels_list_entry *, struct packet_p
 extern void NF_peer_dst_ip_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_ip_proto_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_flow_label_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_fwd_status_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_counters_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -112,15 +115,27 @@ extern void NF_fw_event_handler(struct channels_list_entry *, struct packet_ptrs
 extern void NF_mpls_label_stack_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_label_bottom_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void NF_mpls_vpn_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_srv6_segment_ipv6_list_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_vpn_rd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_mpls_pw_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_path_delay_avg_usec_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_path_delay_min_usec_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_path_delay_max_usec_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_vxlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_nvgre_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_timestamp_end_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_timestamp_arrival_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_timestamp_export_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_sequence_number_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_dst_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_ip_proto_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void NF_tunnel_flow_label_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_version_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_sysid_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void NF_custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -150,6 +165,7 @@ extern void SF_dst_mac_handler(struct channels_list_entry *, struct packet_ptrs
 extern void SF_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_in_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_out_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void SF_cvlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_cos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -203,6 +219,7 @@ extern void SF_mpls_label_bottom_handler(struct channels_list_entry *, struct pa
 extern void SF_mpls_label_stack_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_mpls_pw_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void SF_custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+extern void SF_mpls_vpn_rd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
 extern void pre_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void pre_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -210,12 +227,6 @@ extern void pre_tag_label_handler(struct channels_list_entry *, struct packet_pt
 extern void sampling_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 extern void sfprobe_sampling_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
-#if defined (WITH_GEOIP)
-extern void pm_geoip_init();
-extern void src_host_country_geoip_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-extern void dst_host_country_geoip_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-#endif
-
 #if defined (WITH_GEOIPV2)
 extern void pm_geoipv2_init();
 extern void pm_geoipv2_close();
