@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -38,7 +38,7 @@
 compose_json_handler cjhandler[N_PRIMITIVES];
 
 /* Functions */
-void compose_json(u_int64_t wtc, u_int64_t wtc_2)
+void compose_json(u_int64_t wtc, u_int64_t wtc_2, u_int64_t wtc_3)
 {
   int idx = 0;
 
@@ -97,11 +97,26 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_3 & COUNT_IN_VLAN) {
+    cjhandler[idx] = compose_json_in_vlan;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_OUT_VLAN) {
     cjhandler[idx] = compose_json_out_vlan;
     idx++;
   }
 
+  if (wtc_3 & COUNT_IN_CVLAN) {
+    cjhandler[idx] = compose_json_in_cvlan;
+    idx++;
+  }
+
+  if (wtc_3 & COUNT_OUT_CVLAN) {
+    cjhandler[idx] = compose_json_out_cvlan;
+    idx++;
+  }
+
   if (wtc & COUNT_COS) {
     cjhandler[idx] = compose_json_cos;
     idx++;
@@ -124,22 +139,42 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
   }
 
   if (wtc & COUNT_STD_COMM) {
-    cjhandler[idx] = compose_json_std_comm;
+    if (config.bgp_comms_encode_as_array) {
+      cjhandler[idx] = compose_json_array_std_comm;
+    }
+    else {
+      cjhandler[idx] = compose_json_std_comm;
+    }
     idx++;
   }
 
   if (wtc & COUNT_EXT_COMM) {
-    cjhandler[idx] = compose_json_ext_comm;
+    if (config.bgp_comms_encode_as_array) {
+      cjhandler[idx] = compose_json_array_ext_comm;
+    }
+    else {
+      cjhandler[idx] = compose_json_ext_comm;
+    }
     idx++;
   }
 
   if (wtc_2 & COUNT_LRG_COMM) {
-    cjhandler[idx] = compose_json_lrg_comm;
+    if (config.bgp_comms_encode_as_array) {
+      cjhandler[idx] = compose_json_array_lrg_comm;
+    }
+    else {
+      cjhandler[idx] = compose_json_lrg_comm;
+    }
     idx++;
   }
 
   if (wtc & COUNT_AS_PATH) {
-    cjhandler[idx] = compose_json_as_path;
+    if (config.as_path_encode_as_array) {
+      cjhandler[idx] = compose_json_array_as_path;
+    }
+    else {
+      cjhandler[idx] = compose_json_as_path;
+    }
     idx++;
   }
 
@@ -179,22 +214,42 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
   }
 
   if (wtc & COUNT_SRC_STD_COMM) {
-    cjhandler[idx] = compose_json_src_std_comm;
+    if (config.bgp_comms_encode_as_array) {
+      cjhandler[idx] = compose_json_array_src_std_comm;
+    }
+    else {
+      cjhandler[idx] = compose_json_src_std_comm;
+    }
     idx++;
   }
 
   if (wtc & COUNT_SRC_EXT_COMM) {
-    cjhandler[idx] = compose_json_src_ext_comm;
+    if (config.bgp_comms_encode_as_array) {
+      cjhandler[idx] = compose_json_array_src_ext_comm;
+    }
+    else {
+      cjhandler[idx] = compose_json_src_ext_comm;
+    }
     idx++;
   }
 
   if (wtc_2 & COUNT_SRC_LRG_COMM) {
-    cjhandler[idx] = compose_json_src_lrg_comm;
+    if (config.bgp_comms_encode_as_array) {
+      cjhandler[idx] = compose_json_array_src_lrg_comm;
+    }
+    else {
+      cjhandler[idx] = compose_json_src_lrg_comm;
+    }
     idx++;
   }
 
   if (wtc & COUNT_SRC_AS_PATH) {
-    cjhandler[idx] = compose_json_src_as_path;
+    if (config.as_path_encode_as_array) {
+      cjhandler[idx] = compose_json_array_src_as_path;
+    }
+    else {
+      cjhandler[idx] = compose_json_src_as_path;
+    }
     idx++;
   }
 
@@ -273,18 +328,7 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
-#if defined (WITH_GEOIP)
-  if (wtc_2 & COUNT_SRC_HOST_COUNTRY) {
-    cjhandler[idx] = compose_json_src_host_country;
-    idx++;
-  }
-
-  if (wtc_2 & COUNT_DST_HOST_COUNTRY) {
-    cjhandler[idx] = compose_json_dst_host_country;
-    idx++;
-  }
-#endif
-#if defined (WITH_GEOIPV2)
+#if defined WITH_GEOIPV2
   if (wtc_2 & COUNT_SRC_HOST_COUNTRY) {
     cjhandler[idx] = compose_json_src_host_country;
     idx++;
@@ -326,7 +370,7 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     }
     idx++;
   }
-  
+
   if (wtc_2 & COUNT_FWD_STATUS) {
     if (config.fwd_status_encode_as_string) {
       cjhandler[idx] = compose_json_string_fwd_status;
@@ -347,6 +391,26 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_SRV6_SEG_IPV6_SECTION) {
+    cjhandler[idx] = compose_json_array_srv6_segment_ipv6_list;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_PATH_DELAY_AVG_USEC) {
+    cjhandler[idx] = compose_json_path_delay_avg_usec;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_PATH_DELAY_MIN_USEC) {
+    cjhandler[idx] = compose_json_path_delay_min_usec;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_PATH_DELAY_MAX_USEC) {
+    cjhandler[idx] = compose_json_path_delay_max_usec;
+    idx++;
+  }
+
   if (wtc & COUNT_IP_PROTO) {
     cjhandler[idx] = compose_json_proto;
     idx++;
@@ -357,6 +421,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_3 & COUNT_FLOW_LABEL) {
+    cjhandler[idx] = compose_json_flow_label;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_SAMPLING_RATE) {
     cjhandler[idx] = compose_json_sampling_rate;
     idx++;
@@ -430,13 +499,18 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
   if (wtc_2 & COUNT_TUNNEL_IP_PROTO) {
     cjhandler[idx] = compose_json_tunnel_proto;
     idx++;
-  } 
-    
+  }
+
   if (wtc_2 & COUNT_TUNNEL_IP_TOS) {
     cjhandler[idx] = compose_json_tunnel_tos;
     idx++;
   }
 
+  if (wtc_3 & COUNT_TUNNEL_FLOW_LABEL) {
+    cjhandler[idx] = compose_json_tunnel_flow_label;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_TUNNEL_SRC_PORT) {
     cjhandler[idx] = compose_json_tunnel_src_port;
     idx++;
@@ -462,6 +536,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_3 & COUNT_NVGRE) {
+    cjhandler[idx] = compose_json_nvgre;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_TIMESTAMP_START) {
     cjhandler[idx] = compose_json_timestamp_start;
     idx++;
@@ -580,19 +659,19 @@ void compose_json_src_mac(json_t *obj, struct chained_cache *cc)
 void compose_json_dst_mac(json_t *obj, struct chained_cache *cc)
 {
   char mac[18];
-  
+
   etheraddr_string(cc->primitives.eth_dhost, mac);
   json_object_set_new_nocheck(obj, "mac_dst", json_string(mac));
 }
 
 void compose_json_vlan(json_t *obj, struct chained_cache *cc)
 {
-  if (config.tmp_vlan_legacy) {
-    json_object_set_new_nocheck(obj, "vlan", json_integer((json_int_t)cc->primitives.vlan_id));
-  }
-  else {
-    json_object_set_new_nocheck(obj, "vlan_in", json_integer((json_int_t)cc->primitives.vlan_id));
-  }
+  json_object_set_new_nocheck(obj, "vlan", json_integer((json_int_t)cc->primitives.vlan_id));
+}
+
+void compose_json_in_vlan(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "vlan_in", json_integer((json_int_t)cc->primitives.vlan_id));
 }
 
 void compose_json_out_vlan(json_t *obj, struct chained_cache *cc)
@@ -600,6 +679,16 @@ void compose_json_out_vlan(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "vlan_out", json_integer((json_int_t)cc->primitives.out_vlan_id));
 }
 
+void compose_json_in_cvlan(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "cvlan_in", json_integer((json_int_t)cc->ptun->cvlan_id));
+}
+
+void compose_json_out_cvlan(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "cvlan_out", json_integer((json_int_t)cc->ptun->out_cvlan_id));
+}
+
 void compose_json_cos(json_t *obj, struct chained_cache *cc)
 {
   json_object_set_new_nocheck(obj, "cos", json_integer((json_int_t)cc->primitives.cos));
@@ -891,28 +980,7 @@ void compose_json_dst_port(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "port_dst", json_integer((json_int_t)cc->primitives.dst_port));
 }
 
-#if defined (WITH_GEOIP)
-void compose_json_src_host_country(json_t *obj, struct chained_cache *cc)
-{
-  char empty_string[] = "";
- 
-  if (cc->primitives.src_ip_country.id > 0)
-    json_object_set_new_nocheck(obj, "country_ip_src", json_string(GeoIP_code_by_id(cc->primitives.src_ip_country.id)));
-  else
-    json_object_set_new_nocheck(obj, "country_ip_src", json_string(empty_string));
-}
-
-void compose_json_dst_host_country(json_t *obj, struct chained_cache *cc)
-{
-  char empty_string[] = "";
-
-  if (cc->primitives.dst_ip_country.id > 0)
-    json_object_set_new_nocheck(obj, "country_ip_dst", json_string(GeoIP_code_by_id(cc->primitives.dst_ip_country.id)));
-  else
-    json_object_set_new_nocheck(obj, "country_ip_dst", json_string(empty_string));
-}
-#endif
-#if defined (WITH_GEOIPV2)
+#if defined WITH_GEOIPV2
 void compose_json_src_host_country(json_t *obj, struct chained_cache *cc)
 {
   char empty_string[] = "";
@@ -1010,6 +1078,11 @@ void compose_json_tos(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "tos", json_integer((json_int_t)cc->primitives.tos));
 }
 
+void compose_json_flow_label(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "flow_label", json_integer((json_int_t)cc->primitives.flow_label));
+}
+
 void compose_json_sampling_rate(json_t *obj, struct chained_cache *cc)
 {
   json_object_set_new_nocheck(obj, "sampling_rate", json_integer((json_int_t)cc->primitives.sampling_rate));
@@ -1110,6 +1183,11 @@ void compose_json_tunnel_tos(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "tunnel_tos", json_integer((json_int_t)cc->ptun->tunnel_tos));
 }
 
+void compose_json_tunnel_flow_label(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "tunnel_flow_label", json_integer((json_int_t)cc->ptun->tunnel_flow_label));
+}
+
 void compose_json_tunnel_src_port(json_t *obj, struct chained_cache *cc)
 {
   json_object_set_new_nocheck(obj, "tunnel_port_src", json_integer((json_int_t)cc->ptun->tunnel_src_port));
@@ -1146,6 +1224,11 @@ void compose_json_vxlan(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "vxlan", json_integer((json_int_t)cc->ptun->tunnel_id));
 }
 
+void compose_json_nvgre(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "nvgre", json_integer((json_int_t)cc->ptun->nvgre_tunnel_id));
+}
+
 void compose_json_timestamp_start(json_t *obj, struct chained_cache *cc)
 {
   char tstamp_str[VERYSHORTBUFLEN];
@@ -1343,7 +1426,7 @@ void compose_json_map_label(json_t *obj, struct chained_cache *cc)
   json_t *root_l1 = compose_label_json_data(ptm_ll, ll_size);
 
   json_object_set_new_nocheck(obj, "label", root_l1);
-  
+
   cdada_str_destroy(lbls_cdada);
   cdada_list_destroy(ptm_ll);
 }
@@ -1374,6 +1457,150 @@ void compose_json_string_fwd_status(json_t *obj, struct chained_cache *cc)
   cdada_list_destroy(fwd_status_ll);
 }
 
+void compose_json_array_std_comm(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_STD_COMM, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *std_comm_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(std_comm_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(std_comm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "comms", root_l1);
+
+  cdada_list_destroy(std_comm_ll);
+}
+
+void compose_json_array_src_std_comm(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_SRC_STD_COMM, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *std_comm_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(std_comm_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(std_comm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "comms_src", root_l1);
+
+  cdada_list_destroy(std_comm_ll);
+}
+
+void compose_json_array_ext_comm(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_EXT_COMM, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *ext_comm_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(ext_comm_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(ext_comm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "ecomms", root_l1);
+
+  cdada_list_destroy(ext_comm_ll);
+}
+
+void compose_json_array_src_ext_comm(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_SRC_EXT_COMM, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *ext_comm_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(ext_comm_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(ext_comm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "ecomms_src", root_l1);
+
+  cdada_list_destroy(ext_comm_ll);
+}
+
+void compose_json_array_lrg_comm(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_LRG_COMM, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *lrg_comm_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(lrg_comm_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(lrg_comm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "lcomms", root_l1);
+
+  cdada_list_destroy(lrg_comm_ll);
+}
+
+void compose_json_array_src_lrg_comm(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_SRC_LRG_COMM, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *lrg_comm_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(lrg_comm_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(lrg_comm_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "lcomms_src", root_l1);
+
+  cdada_list_destroy(lrg_comm_ll);
+}
+
+void compose_json_array_as_path(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_AS_PATH, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *as_path_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(as_path_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(as_path_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "as_apth", root_l1);
+
+  cdada_list_destroy(as_path_ll);
+}
+
+void compose_json_array_src_as_path(json_t *obj, struct chained_cache *cc)
+{
+  char empty_string[] = "", *str_ptr;
+
+  vlen_prims_get(cc->pvlen, COUNT_INT_SRC_AS_PATH, &str_ptr);
+  if (!str_ptr) str_ptr = empty_string;
+
+  /* linked-list creation */
+  cdada_list_t *as_path_ll = generic_delim_str_to_linked_list(str_ptr, GENERIC_STR_DELIM);;
+  size_t ll_size = cdada_list_size(as_path_ll);
+
+  json_t *root_l1 = compose_str_linked_list_to_json_array_data(as_path_ll, ll_size);
+
+  json_object_set_new_nocheck(obj, "as_apth_src", root_l1);
+
+  cdada_list_destroy(as_path_ll);
+}
+
 void compose_json_array_mpls_label_stack(json_t *obj, struct chained_cache *cc)
 {
   char *label_stack_ptr = NULL;
@@ -1385,6 +1612,17 @@ void compose_json_array_mpls_label_stack(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "mpls_label_stack", root_l1);
 }
 
+void compose_json_array_srv6_segment_ipv6_list(json_t *obj, struct chained_cache *cc)
+{
+  struct host_addr *list_ptr = NULL;
+  int list_len = 0;
+
+  list_len = vlen_prims_get(cc->pvlen, COUNT_INT_SRV6_SEG_IPV6_SECTION, (char **) &list_ptr);
+  json_t *root_l1 = compose_srv6_segment_ipv6_list_json_data((struct host_addr *)list_ptr, list_len);
+
+  json_object_set_new_nocheck(obj, "srv6_seg_ipv6_list", root_l1);
+}
+
 json_t *compose_label_json_data(cdada_list_t *ll, int ll_size)
 {
   ptm_label lbl;
@@ -1489,8 +1727,84 @@ json_t *compose_mpls_label_stack_json_data(u_int32_t *label_stack, int ls_len)
     strncat(label_idx_buf, label_buf, (MAX_MPLS_LABEL_IDX_LEN - max_mpls_label_idx_len_dec));
     max_mpls_label_idx_len_dec = (strlen(idx_buf) + strlen("-") + strlen(label_buf) + 3);
     j_str_tmp = json_string(label_idx_buf);
-    json_array_append(root, j_str_tmp); 
+    json_array_append(root, j_str_tmp);
   }
 
   return root;
 }
+
+json_t *compose_srv6_segment_ipv6_list_json_data(struct host_addr *ipv6_list, int list_len)
+{
+  const int MAX_IDX_LEN = 2 /* index */ + 1 /* sep */ + 1 /* term */;
+  const int MAX_IDX_SID_LEN = (MAX_IDX_LEN + INET6_ADDRSTRLEN);
+
+  char ipv6_str[INET6_ADDRSTRLEN];
+  char idx_str[MAX_IDX_LEN];
+  char idx_ipv6_str[MAX_IDX_SID_LEN];
+  u_int8_t list_elems = 0;
+
+  if (!(list_len % sizeof(struct host_addr))) {
+    list_elems = (list_len / sizeof(struct host_addr));
+  }
+  else {
+    return NULL;
+  }
+
+  json_t *root = json_array();
+  json_t *j_str_tmp = NULL;
+
+  size_t idx_0;
+  for (idx_0 = 0; idx_0 < list_elems; idx_0++) {
+
+    memset(ipv6_str, 0, sizeof(ipv6_str));
+    addr_to_str(ipv6_str, &ipv6_list[idx_0]);
+
+    memset(idx_str, 0, sizeof(idx_str));
+    snprintf(idx_str, 3, "%zu", idx_0);
+    strcat(idx_str, "-");
+
+    memset(idx_ipv6_str, 0, sizeof(idx_ipv6_str));
+    strcat(idx_ipv6_str, idx_str);
+    strcat(idx_ipv6_str, ipv6_str);
+
+    j_str_tmp = json_string(idx_ipv6_str);
+    json_array_append(root, j_str_tmp);
+  }
+
+  return root;
+}
+
+json_t *compose_str_linked_list_to_json_array_data(cdada_list_t *ll, int ll_size)
+{
+  generic_delim_string delim_s = {0};
+
+  json_t *root = json_array();
+  json_t *j_str_tmp = NULL;
+
+  size_t idx_0;
+  for (idx_0 = 0; idx_0 < ll_size; idx_0++) {
+    memset(&delim_s, 0, sizeof(delim_s));
+    cdada_list_get(ll, idx_0, &delim_s);
+    if (strncmp(delim_s.delim_str, "NULL", (strlen(delim_s.delim_str) - 1)) != 0) {
+      j_str_tmp = json_string(delim_s.delim_str);
+      json_array_append(root, j_str_tmp);
+    }
+  }
+
+  return root;
+}
+
+void compose_json_path_delay_avg_usec(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "path_delay_avg_usec", json_integer((json_int_t)cc->pmpls->path_delay_avg_usec));
+}
+
+void compose_json_path_delay_min_usec(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "path_delay_min_usec", json_integer((json_int_t)cc->pmpls->path_delay_min_usec));
+}
+
+void compose_json_path_delay_max_usec(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "path_delay_max_usec", json_integer((json_int_t)cc->pmpls->path_delay_max_usec));
+}
