@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -44,13 +44,15 @@ extern int protocols_number;
 static const struct _primitives_matrix_struct _primitives_matrix[] = {
   /* primitive, pmacctd, uacctd, nfacctd, sfacctd, pmtelemetryd, pmbgpd, pmbmpd */
   {"L2", 1, 1, 1, 1, 0, 0, 0, ""}, 
-  {"cos", 1, 0, 1, 1, 0, 0, 0, "Ethernet CoS, 802.1P"},
-  {"etype", 1, 0, 1, 1, 0, 0, 0, "Ethernet Ethertype"},
+  {"cos", 1, 1, 1, 1, 0, 0, 0, "Ethernet CoS, 802.1P"},
+  {"etype", 1, 1, 1, 1, 0, 0, 0, "Ethernet Ethertype"},
   {"src_mac", 1, 1, 1, 1, 0, 0, 0, "Source MAC address"},
   {"dst_mac", 1, 1, 1, 1, 0, 0, 0, "Destination MAC address"},
   {"vlan", 1, 1, 1, 1, 0, 0, 0, "Ethernet VLAN, 802.1Q"},
   {"in_vlan", 0, 0, 1, 1, 0, 0, 0, "Input Ethernet VLAN, 802.1Q"},
   {"out_vlan", 0, 0, 1, 1, 0, 0, 0, "Output Ethernet VLAN, 802.1Q"},
+  {"in_cvlan", 1, 1, 1, 1, 0, 0, 0, "Input Customer Ethernet VLAN"},
+  {"out_cvlan", 0, 0, 1, 0, 0, 0, 0, "Output Customer Ethernet VLAN"},
   {"L3", 1, 1, 1, 1, 0, 0, 0, ""}, 
   {"src_host", 1, 1, 1, 1, 0, 0, 0, "Source IPv4/IPv6 address"}, 
   {"dst_host", 1, 1, 1, 1, 0, 0, 0, "Destination IPv4/IPv6 address"},
@@ -91,15 +93,17 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"post_nat_dst_port", 0, 0, 1, 0, 0, 0, 0, "Destination TCP/UDP port after NAT translation"},
   {"TUNNEL", 1, 1, 1, 1, 0, 0, 0, ""}, 
   {"vxlan", 1, 1, 1, 1, 0, 0, 0, "VXLAN Network Identifier"},
+  {"nvgre", 0, 0, 1, 0, 0, 0, 0, "NVGRE Tenant Network Identifier"},
   {"tunnel_src_mac", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner Source MAC address"},
   {"tunnel_dst_mac", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner Destination MAC address"},
-  {"tunnel_src_host", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner Source IPv4/IPv6 address"},
-  {"tunnel_dst_host", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner Destination IPv4/IPv6 address"},
-  {"tunnel_proto", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner IP protocol"},
-  {"tunnel_tos", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner IP ToS"},
-  {"tunnel_src_port", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner Source TCP/UDP port"},
-  {"tunnel_dst_port", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner Destination TCP/UDP port"},
-  {"tunnel_tcpflags", 1, 1, 0, 1, 0, 0, 0, "Tunnel inner TCP flags"},
+  {"tunnel_src_host", 1, 1, 1, 1, 0, 0, 0, "Tunnel Source IPv4/IPv6 address"},
+  {"tunnel_dst_host", 1, 1, 1, 1, 0, 0, 0, "Tunnel Destination IPv4/IPv6 address"},
+  {"tunnel_proto", 1, 1, 1, 1, 0, 0, 0, "Tunnel IP protocol"},
+  {"tunnel_tos", 1, 1, 1, 1, 0, 0, 0, "Tunnel IP ToS"},
+  {"tunnel_src_port", 1, 1, 1, 1, 0, 0, 0, "Tunnel Source TCP/UDP port"},
+  {"tunnel_dst_port", 1, 1, 1, 1, 0, 0, 0, "Tunnel Destination TCP/UDP port"},
+  {"tunnel_tcpflags", 1, 1, 1, 1, 0, 0, 0, "Tunnel TCP flags"},
+  {"tunnel_flow_label", 0, 0, 1, 0, 0, 0, 0, "Tunnel IPv6 flow label"},
   {"MPLS", 1, 1, 1, 1, 0, 0, 0, ""}, 
   {"mpls_label_bottom", 1, 1, 1, 0, 0, 0, 0, "Bottom MPLS label"},
   {"mpls_label_top", 1, 1, 1, 0, 0, 0, 0, "Top MPLS label"},
@@ -109,6 +113,7 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"MISC", 1, 1, 1, 1, 0, 0, 0, ""}, 
   {"class", 1, 1, 1, 1, 0, 0, 0, "L7 protocol classification"},
   {"flows", 0, 0, 1, 0, 0, 0, 0, "IP flows"},
+  {"flow_label", 0, 0, 1, 0, 0, 0, 0, "IPv6 flow label"},
   {"src_host_country", 1, 1, 1, 1, 0, 0, 0, "Source IP address GeoIP resolution: country"},
   {"dst_host_country", 1, 1, 1, 1, 0, 0, 0, "Destination IP address GeoIP resolution: country"},
   {"src_host_pocode", 1, 1, 1, 1, 0, 0, 0, "Source IP address GeoIP resolution: postal code"},
@@ -119,7 +124,7 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"out_iface", 0, 1, 1, 1, 0, 0, 0, "Output interface, SNMP ifIndex"}, 
   {"peer_src_ip", 0, 0, 1, 1, 0, 0, 0, "IP address or identificator of telemetry exporting device"},
   {"sampling_rate", 1, 1, 1, 1, 0, 0, 0, "Sampling rate"},
-  {"sampling_direction", 0, 0, 1, 0, 0, 0, 0, "Sampling direction (ie. ingress vs egress)"},
+  {"sampling_direction", 0, 0, 1, 1, 0, 0, 0, "Sampling direction (ie. ingress vs egress)"},
   {"tag", 1, 1, 1, 1, 0, 0, 0, "Numeric tag, ie. as result of pre_tag_map evaluation"},
   {"tag2", 1, 1, 1, 1, 0, 0, 0, "Numeric tag, ie. as result of pre_tag_map evaluation"},
   {"label", 1, 1, 1, 1, 0, 0, 0, "String label, ie. as result of pre_tag_map evaluation"},
@@ -134,6 +139,9 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"timestamp_end", 0, 0, 1, 0, 0, 0, 0, "Flow end time"},
   {"timestamp_arrival", 1, 1, 1, 1, 0, 0, 0, "Observation time at the collector"},
   {"timestamp_export", 0, 0, 1, 0, 0, 0, 0, "Observation time at the exporter"},
+  {"path_delay_avg_usec", 0, 0, 1, 0, 0, 0, 0, "Average On-Path Telemetry measured delay (usec)"},
+  {"path_delay_min_usec", 0, 0, 1, 0, 0, 0, 0, "Minimum On-Path Telemetry measured delay (usec)"},
+  {"path_delay_max_usec", 0, 0, 1, 0, 0, 0, 0, "Maximum On-Path Telemetry measured delay (usec)"},
   {"", 0, 0, 0, 0, 0, 0, 0, ""}
 };
 
@@ -281,7 +289,7 @@ static const struct _protocols_struct _protocols[] = {
   {"140", 140}, 
   {"141", 141},
   {"142", 142}, 
-  {"143", 143},
+  {"ethernet", 143},
   {"144", 144},
   {"145", 145},
   {"146", 146},
@@ -432,6 +440,7 @@ static const char __attribute__((unused)) *bgp_rd_origin[] = {
   "bgp",
   "bmp",
   "flow",
+  "map",
   ""
 };
 
@@ -480,6 +489,7 @@ static const struct _id_to_string_struct _primitives_map[] = {
   {COUNT_INT_TAG, "tag"},
   {COUNT_INT_VLAN, "vlan"},
   {COUNT_INT_IP_TOS, "tos"},
+  {COUNT_INT_FLOW_LABEL, "flow_label"},
   {COUNT_INT_NONE, "none"},
   {COUNT_INT_SRC_AS, "src_as"},
   {COUNT_INT_DST_AS, "dst_as"},
@@ -537,6 +547,7 @@ static const struct _id_to_string_struct _primitives_map[] = {
   {COUNT_INT_TUNNEL_DST_HOST, "tunnel_dst_host"},
   {COUNT_INT_TUNNEL_IP_PROTO, "tunnel_proto"},
   {COUNT_INT_TUNNEL_IP_TOS, "tunnel_tos"},
+  {COUNT_INT_TUNNEL_FLOW_LABEL, "tunnel_flow_label"},
   {COUNT_INT_NDPI_CLASS, "class"},
   {COUNT_INT_SRC_HOST_COORDS, "src_host_coords"},
   {COUNT_INT_DST_HOST_COORDS, "dst_host_coords"},
@@ -545,6 +556,7 @@ static const struct _id_to_string_struct _primitives_map[] = {
   {COUNT_INT_SRC_ROA, "src_roa"},
   {COUNT_INT_DST_ROA, "dst_roa"},
   {COUNT_INT_VXLAN, "vxlan"},
+  {COUNT_INT_NVGRE, "nvgre"},
   {COUNT_INT_TUNNEL_SRC_MAC, "tunnel_src_mac"},
   {COUNT_INT_TUNNEL_DST_MAC, "tunnel_dst_mac"},
   {COUNT_INT_TUNNEL_SRC_PORT, "tunnel_src_port"},
