@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -20,14 +20,14 @@
 */
 
 /* defines */
-#define ARGS_NFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:MuRVaAE:I:WZ:Y:"
-#define ARGS_SFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:MuRVaAE:I:WZ:Y:"
-#define ARGS_PMACCTD "n:NdDhP:b:f:F:c:i:I:m:p:r:s:S:o:t:O:MuwWZ:Y:L:RVazAE:"
-#define ARGS_UACCTD "n:NdDhP:b:f:F:c:m:p:r:s:S:o:t:O:MuRg:L:VaAE:"
-#define ARGS_PMTELEMETRYD "hVL:u:t:Z:f:dDS:F:o:O:i:"
-#define ARGS_PMBGPD "hVL:l:f:dDS:F:o:O:i:gm:"
-#define ARGS_PMBMPD "hVL:l:f:dDS:F:o:O:i:I:Z:Y:T:"
-#define ARGS_PMACCT "hSsc:Cetm:p:P:M:arN:n:lT:O:E:uVUiI0"
+#define ARGS_NFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:MuRVaAE:I:WZ:Y:T:"
+#define ARGS_SFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:MuRVaAE:I:WZ:Y:T:"
+#define ARGS_PMACCTD "n:NdDhP:b:f:F:c:i:I:m:p:r:s:S:o:t:O:MuwWZ:Y:L:RVazAE:T:"
+#define ARGS_UACCTD "n:NdDhP:b:f:F:c:m:p:r:s:S:o:t:O:MuRg:L:VaAE:T:"
+#define ARGS_PMTELEMETRYD "hVL:u:t:f:dDS:F:o:O:i:T:"
+#define ARGS_PMBGPD "hVL:l:f:dDS:F:o:O:i:gm:T:"
+#define ARGS_PMBMPD "hVL:l:f:dDS:F:o:O:i:I:Z:Y:R:T:"
+#define ARGS_PMACCT "hSsc:Cet:p:P:M:arN:n:lT:O:E:uVUiI0"
 #define N_PRIMITIVES 128
 #define N_FUNCS 10 
 #define MAX_N_PLUGINS 32
@@ -233,7 +233,7 @@
 #define COUNT_INT_TIMESTAMP_ARRIVAL	0x0002000000000800ULL
 #define COUNT_INT_MPLS_LABEL_TOP	0x0002000000001000ULL
 #define COUNT_INT_MPLS_LABEL_BOTTOM	0x0002000000002000ULL
-/* #define XXX: available 		0x0002000000004000ULL */
+#define COUNT_INT_SRV6_SEG_IPV6_SECTION	0x0002000000004000ULL
 #define COUNT_INT_LABEL			0x0002000000008000ULL
 #define COUNT_INT_EXPORT_PROTO_SEQNO	0x0002000000010000ULL
 #define COUNT_INT_EXPORT_PROTO_VERSION  0x0002000000020000ULL
@@ -264,8 +264,19 @@
 #define COUNT_INT_FW_EVENT		0x0002020000000000ULL
 #define COUNT_INT_TUNNEL_TCPFLAGS	0x0002040000000000ULL
 #define COUNT_INT_OUT_VLAN		0x0002080000000000ULL
+#define COUNT_INT_PATH_DELAY_AVG_USEC	0x0002100000000000ULL
+#define COUNT_INT_PATH_DELAY_MIN_USEC	0x0002200000000000ULL
+#define COUNT_INT_PATH_DELAY_MAX_USEC	0x0002400000000000ULL
 #define COUNT_INT_CUSTOM_PRIMITIVES	0x0002800000000000ULL
 
+/* internal: third registry, ie. what_to_count_3, aggregation_3, etc. */
+#define COUNT_INT_FLOW_LABEL		0x0003000000000001ULL
+#define COUNT_INT_TUNNEL_FLOW_LABEL	0x0003000000000002ULL
+#define COUNT_INT_IN_VLAN		0x0003000000000004ULL
+#define COUNT_INT_IN_CVLAN		0x0003000000000008ULL
+#define COUNT_INT_OUT_CVLAN		0x0003000000000010ULL
+#define COUNT_INT_NVGRE			0x0003000000000020ULL
+
 #define COUNT_INDEX_MASK	0xFFFF
 #define COUNT_INDEX_CP		0xFFFF000000000000ULL  /* index 0xffff reserved to custom primitives */
 #define COUNT_REGISTRY_MASK	0x0000FFFFFFFFFFFFULL
@@ -336,6 +347,7 @@
 #define COUNT_TIMESTAMP_ARRIVAL		(COUNT_INT_TIMESTAMP_ARRIVAL & COUNT_REGISTRY_MASK)
 #define COUNT_MPLS_LABEL_TOP		(COUNT_INT_MPLS_LABEL_TOP & COUNT_REGISTRY_MASK)
 #define COUNT_MPLS_LABEL_BOTTOM		(COUNT_INT_MPLS_LABEL_BOTTOM & COUNT_REGISTRY_MASK)
+#define COUNT_SRV6_SEG_IPV6_SECTION	(COUNT_INT_SRV6_SEG_IPV6_SECTION & COUNT_REGISTRY_MASK)
 #define COUNT_LABEL			(COUNT_INT_LABEL & COUNT_REGISTRY_MASK)
 #define COUNT_EXPORT_PROTO_SEQNO	(COUNT_INT_EXPORT_PROTO_SEQNO & COUNT_REGISTRY_MASK)
 #define COUNT_EXPORT_PROTO_VERSION	(COUNT_INT_EXPORT_PROTO_VERSION & COUNT_REGISTRY_MASK)
@@ -365,7 +377,18 @@
 #define COUNT_FW_EVENT			(COUNT_INT_FW_EVENT & COUNT_REGISTRY_MASK)
 #define COUNT_TUNNEL_TCPFLAGS		(COUNT_INT_TUNNEL_TCPFLAGS & COUNT_REGISTRY_MASK)
 #define COUNT_OUT_VLAN			(COUNT_INT_OUT_VLAN & COUNT_REGISTRY_MASK)
+#define COUNT_PATH_DELAY_AVG_USEC	(COUNT_INT_PATH_DELAY_AVG_USEC & COUNT_REGISTRY_MASK)
+#define COUNT_PATH_DELAY_MIN_USEC	(COUNT_INT_PATH_DELAY_MIN_USEC & COUNT_REGISTRY_MASK)
+#define COUNT_PATH_DELAY_MAX_USEC	(COUNT_INT_PATH_DELAY_MAX_USEC & COUNT_REGISTRY_MASK)
 #define COUNT_CUSTOM_PRIMITIVES		(COUNT_INT_CUSTOM_PRIMITIVES & COUNT_REGISTRY_MASK)
+
+/* external: third registry, ie. what_to_count_3, aggregation_3, etc. */
+#define COUNT_FLOW_LABEL		(COUNT_INT_FLOW_LABEL & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_FLOW_LABEL		(COUNT_INT_TUNNEL_FLOW_LABEL & COUNT_REGISTRY_MASK)
+#define COUNT_IN_VLAN			(COUNT_INT_IN_VLAN & COUNT_REGISTRY_MASK)
+#define COUNT_IN_CVLAN			(COUNT_INT_IN_CVLAN & COUNT_REGISTRY_MASK)
+#define COUNT_OUT_CVLAN			(COUNT_INT_OUT_CVLAN & COUNT_REGISTRY_MASK)
+#define COUNT_NVGRE			(COUNT_INT_NVGRE & COUNT_REGISTRY_MASK)
 /* PRIMITIVES DEFINITION: END */
 
 /* BYTES and PACKETS are used into templates; we let their values to
@@ -426,7 +449,8 @@
 #define BGP_SRC_PRIMITIVES_MAP	0x00000002
 #define BGP_SRC_PRIMITIVES_BGP	0x00000004
 
-#define BGP_ASPATH_HASH_PATHID	0x00000000
+#define BGP_ASPATH_HASH_PATHID	   0x00000000
+#define BGP_ASPATH_HASH_MPLSVPNRD  0x00000001
 
 #define PRINT_OUTPUT_FORMATTED	0x00000001
 #define PRINT_OUTPUT_CSV	0x00000002
@@ -483,6 +507,10 @@
 #define DYN_STR_WRITER_ID		7
 #define DYN_STR_MAX			7
 
+#define DRY_RUN_UNKNOWN			0
+#define DRY_RUN_CONFIG			1
+#define DRY_RUN_SETUP			2
+
 typedef u_int32_t pm_class_t;
 typedef u_int64_t pm_id_t;
 typedef u_int64_t pm_cfgreg_t;
@@ -558,6 +586,9 @@ typedef u_int64_t pm_counter_t;
 #define PM_FTYPE_VLAN_MPLS		15 /* PM_FTYPE_MPLS + PM_FTYPE_VLAN */
 #define PM_FTYPE_VLAN_MPLS_IPV4		17 /* PM_FTYPE_VLAN_MPLS + PM_FTYPE_IPV4 */
 #define PM_FTYPE_VLAN_MPLS_IPV6		18 /* PM_FTYPE_VLAN_MPLS + PM_FTYPE_IPV6 */
+#define PM_FTYPE_SRV6			20
+#define PM_FTYPE_SRV6_IPV4		22 /* PM_FTYPE_SRV6 + PM_FTYPE_IPV4 */
+#define PM_FTYPE_SRV6_IPV6		23 /* PM_FTYPE_SRV6 + PM_FTYPE_IPV6 */
 #define PM_FTYPE_TRAFFIC_MAX		99  /* temporary: re-coding needed */
 
 /* flow type: NetFlow/IPFIX extended code-points */
