@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -85,16 +85,9 @@
 #endif
 
 #include "pmacct-version.h"
-#include "pmacct-build.h"
 #include "pmacct-defines.h"
 
-#if defined (WITH_GEOIP)
-#include <GeoIP.h>
-#if defined (WITH_GEOIPV2)
-#error "--enable-geoip and --enable-geoipv2 are mutually exclusive"
-#endif
-#endif
-#if defined (WITH_GEOIPV2)
+#if defined WITH_GEOIPV2
 #include <maxminddb.h>
 #endif
 
@@ -460,6 +453,7 @@ extern struct pm_pcap_devices devices, bkp_devices;
 extern struct pm_pcap_interfaces pm_pcap_if_map, pm_bkp_pcap_if_map;
 extern struct pcap_stat ps;
 extern struct sigaction sighandler_action;
+extern int bmp_bgp_forwarding;  /* flag used by BGP-BMP-HA feature if enabled */
 
 extern char pmacctd_globstr[];
 extern char nfacctd_globstr[];
@@ -468,4 +462,5 @@ extern char uacctd_globstr[];
 extern char pmtele_globstr[];
 extern char pmbgpd_globstr[];
 extern char pmbmpd_globstr[];
+
 #endif /* _PMACCT_H_ */
