@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -21,6 +21,7 @@
 
 /* includes */
 #include "pmacct.h"
+#include "pmacct-build.h"
 #include "bgp/bgp.h"
 #include "bmp/bmp.h"
 #include "pmbmpd.h"
@@ -30,6 +31,9 @@
 #include "pkt_handlers.h"
 #include "ip_flow.h"
 #include "net_aggr.h"
+#ifdef WITH_REDIS
+#include "ha.h"
+#endif
 
 /* Functions */
 void usage_daemon(char *prog_name)
@@ -46,7 +50,7 @@ void usage_daemon(char *prog_name)
   printf("  -I  \tRead packets from the specified savefile\n");
   printf("  -Z  \tReading from a savefile, sleep the given amount of seconds at startup and between replays\n");
   printf("  -Y  \tReading from a savefile, replay the number of times specified\n");
-  printf("  -T  \tReading from a savefile, apply the filter specified with tcpdump syntax, ie. \"dst port 1790\"\n");
+  printf("  -R  \tReading from a savefile, apply the filter specified with tcpdump syntax, ie. \"dst port 1790\"\n");
   printf("  -f  \tLoad configuration from the specified file\n");
   printf("  -D  \tDaemonize\n");
   printf("  -d  \tEnable debug\n");
@@ -55,6 +59,7 @@ void usage_daemon(char *prog_name)
   printf("  -o  \tOutput file to log real-time BMP messages\n");
   printf("  -O  \tOutput file to dump BMP data and generated RIBs at regular time intervals\n");
   printf("  -i  \tInterval, in secs, to write to the dump output file (supplied by -O)\n");
+  printf("  -T  \t[ config | setup ]\n\tPerform a dry run\n");
   printf("\n");
   printf("For examples, see:\n");
   printf("  https://github.com/pmacct/pmacct/blob/master/QUICKSTART or\n");
@@ -75,7 +80,7 @@ int main(int argc,char **argv, char **envp)
   int errflag, cp;
 
 #ifdef WITH_REDIS
-  struct p_redis_host redis_host;
+  struct p_redis_host redis_host, redis_ha_host;
 #endif
 
 #if defined HAVE_MALLOPT
@@ -132,7 +137,7 @@ int main(int argc,char **argv, char **envp)
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
       break;
-    case 'T':
+    case 'R':
       strlcpy(cfg_cmdline[rows], "pcap_filter: ", SRVBUFLEN);
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
@@ -174,6 +179,11 @@ int main(int argc,char **argv, char **envp)
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
       break;
+    case 'T':
+      strlcpy(cfg_cmdline[rows], "dry_run: ", SRVBUFLEN);
+      strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
+      rows++;
+      break;
     case 'h':
       usage_daemon(argv[0]);
       exit(0);
@@ -212,6 +222,17 @@ int main(int argc,char **argv, char **envp)
     list = list->next;
   }
 
+  if (config.dry_run == DRY_RUN_CONFIG) {
+    printf("INFO ( %s/%s ): Dry run 'config'. Exiting ..\n", config.name, config.type);
+    exit(0);
+  }
+  else if (config.dry_run  == DRY_RUN_SETUP) {
+    if (config.daemon) {
+      printf("INFO ( %s/%s ): Dry run 'setup'. Not daemonizing.\n", config.name, config.type);
+      config.daemon = FALSE;
+    }
+  }
+
   if (config.files_umask) umask(config.files_umask);
 
   initsetproctitle(argc, argv, envp);
@@ -299,17 +320,52 @@ int main(int argc,char **argv, char **envp)
   sighandler_action.sa_handler = PM_sigalrm_noop_handler;
   sigaction(SIGALRM, &sighandler_action, NULL);
 
+#ifdef WITH_REDIS
+  /* Signals for BMP-BGP-HA feature */
+  if (config.bgp_bmp_daemon_ha) {
+    /* reset the local timestamp of the collector (SIGNAL=34)*/
+    sighandler_action.sa_handler = bmp_bgp_ha_regenerate_timestamp;
+    sigaction(SIGRTMIN, &sighandler_action, NULL);
+
+    /* set collector to forced active mode (SIGNAL=35)*/
+    sighandler_action.sa_handler = bmp_bgp_ha_set_to_active;
+    sigaction(SIGRTMIN + 1, &sighandler_action, NULL);
+
+    /* set collector to forced stand-by mode (SIGNAL=36)*/
+    sighandler_action.sa_handler = bmp_bgp_ha_set_to_standby;
+    sigaction(SIGRTMIN + 2, &sighandler_action, NULL);
+
+    /* set collector back to timestamp-based (i.e. non-forced) mode (SIGNAL=37)*/
+    sighandler_action.sa_handler = bmp_bgp_ha_set_to_normal;
+    sigaction(SIGRTMIN + 3, &sighandler_action, NULL);
+  }
+#endif
+
   if (!config.bmp_daemon_port) config.bmp_daemon_port = BMP_TCP_PORT;
 
 #ifdef WITH_REDIS
+  /* Kicking off redis-reladed thread(s) */
   if (config.redis_host) {
     char log_id[SHORTBUFLEN];
 
     snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+
+    if (config.bgp_bmp_daemon_ha) {
+      /* If BMP-BGP-HA feature is enabled, redirect redis thread */
+      p_redis_init(&redis_ha_host, log_id, p_redis_thread_bmp_bgp_ha_handler);
+    }
+
     p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_core_handler);
   }
 #endif
 
+#ifdef WITH_REDIS
+  if (config.bgp_bmp_daemon_ha) {
+    /* Kicking off BMP-BGP-HA feature (BMP-BGP Daemon High Availability) */
+    bmp_bgp_ha_main();
+  }
+#endif
+
   bmp_prepare_daemon();
   skinny_bmp_daemon();
 
