@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -438,11 +438,12 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 		      memcpy(&recirc_e, &tmp.e[tmp.num], sizeof(struct id_entry));
 
 		      /*
-			If indexing is enabled and no address family is specified for 'ip'
-			(ie. 'ip' is likely not specified), we will not include it as part
-			of the index hash serializer anyway; we can skip recirculation.
+			Recirculate by default in order to ensure correctness of maps;
+			if this can be skipped, ie. v4-only or no-ip-only map, then the
+			'pre_tag_map_dont_recirculate' config knob can be used to save
+			memory.
 		      */
-		      if (!config.maps_index) {
+		      if (!config.pre_tag_map_dont_recirculate) {
 			recirculate = TRUE;
 		      }
 		    }
@@ -471,7 +472,12 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
                   tmp.num++;
 
 		  if (recirculate) {
-		    if (tmp.num < map_entries) goto recirculate_ipv6;
+		    if (tmp.num < map_entries) {
+		      tmp.e[tmp.num].key.agent_ip.a.family = FALSE;
+		      tmp.e[tmp.num].key.agent_mask.family = FALSE;
+
+		      goto recirculate_ipv6;
+		    }
 		  }
                 }
 	        /* if any required field is missing and other errors have been signalled
@@ -532,7 +538,7 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
                         config.name, config.type, filename, tot_lines);
               }
               else if (acct_type == MAP_FLOW_TO_RD) {
-                if (!err && tmp.e[tmp.num].id && tmp.e[tmp.num].key.agent_ip.a.family) {
+                if (!err && tmp.e[tmp.num].key.agent_ip.a.family) {
                   int j;
 
                   for (j = 0; tmp.e[tmp.num].func[j]; j++);
@@ -586,7 +592,7 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
       t->ipv4_base = &t->e[x];
       t->flags = tmp.flags;
       {
-	pm_id_t pm_cdada_map_container;
+	char pm_cdada_map_container[MAX_LABEL_LEN];
 
         t->label_map_v4 = cdada_map_create(pm_cdada_map_container);
         t->label_map_v6 = cdada_map_create(pm_cdada_map_container);
@@ -604,10 +610,14 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	  if (t->e[x].entry_label) {
 	    int ret;
 	    pm_id_t *idx_value;
+	    char entry_label[MAX_LABEL_LEN];
 
-	    ret = cdada_map_find(t->label_map_v4, t->e[x].entry_label, (void **) &idx_value);
+	    memset(entry_label, 0, MAX_LABEL_LEN);
+	    strcpy(entry_label, t->e[x].entry_label);
+
+	    ret = cdada_map_find(t->label_map_v4, entry_label, (void **) &idx_value);
 	    if (ret == CDADA_E_NOT_FOUND) {
-	      ret = cdada_map_insert(t->label_map_v4, t->e[x].entry_label, &t->e[x].pos);
+	      ret = cdada_map_insert(t->label_map_v4, entry_label, &t->e[x].pos);
 	      if (ret != CDADA_SUCCESS) {
 		Log(LOG_ERR, "ERROR ( %s/%s ): [%s] unable to insert in Label Map v4. Exiting.\n", config.name, config.type, t->filename);
 		exit_gracefully(1);
@@ -629,10 +639,14 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	  if (t->e[x].entry_label) {
 	    int ret;
 	    pm_id_t *idx_value;
+            char entry_label[MAX_LABEL_LEN];
 
-	    ret = cdada_map_find(t->label_map_v6, t->e[x].entry_label, (void **) &idx_value);
+            memset(entry_label, 0, MAX_LABEL_LEN);
+            strcpy(entry_label, t->e[x].entry_label);
+
+	    ret = cdada_map_find(t->label_map_v6, entry_label, (void **) &idx_value);
 	    if (ret == CDADA_E_NOT_FOUND) {
-	      ret = cdada_map_insert(t->label_map_v6, t->e[x].entry_label, &t->e[x].pos);
+	      ret = cdada_map_insert(t->label_map_v6, entry_label, &t->e[x].pos);
 	      if (ret != CDADA_SUCCESS) {
 		Log(LOG_ERR, "ERROR ( %s/%s ): [%s] unable to insert in Label Map v6. Exiting.\n", config.name, config.type, t->filename);
 		exit_gracefully(1);
@@ -653,14 +667,20 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 
 	  /* honouring reserved labels (ie. "next"). Then resolving unknown labels */
 	  if (!strcmp(ptr->jeq.label, "next")) {
-	    ptr->jeq.ptr = ptr+1;
-	    label_solved = TRUE;
+	    if (x < (t->ipv4_num - 1)) {
+	      ptr->jeq.ptr = ptr+1;
+	      label_solved = TRUE;
+	    }
 	  }
 	  else {
             int ret;
             pm_id_t *idx_value;
+            char entry_label[MAX_LABEL_LEN];
 
-            ret = cdada_map_find(t->label_map_v4, ptr->jeq.label, (void **) &idx_value);
+            memset(entry_label, 0, MAX_LABEL_LEN);
+            strcpy(entry_label, ptr->jeq.label);
+
+            ret = cdada_map_find(t->label_map_v4, entry_label, (void **) &idx_value);
 	    if (ret == CDADA_SUCCESS) {
 	      if ((*idx_value) > ptr->pos) {
 		ptr->jeq.ptr = &t->e[(*idx_value)];
@@ -680,10 +700,14 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
         if (ptr->jeq.label) {
           int ret;
           pm_id_t *idx_value;
+          char entry_label[MAX_LABEL_LEN];
+
+          memset(entry_label, 0, MAX_LABEL_LEN);
+          strcpy(entry_label, ptr->jeq.label);
 
           label_solved = FALSE;
 
-          ret = cdada_map_find(t->label_map_v6, ptr->jeq.label, (void **) &idx_value);
+          ret = cdada_map_find(t->label_map_v6, entry_label, (void **) &idx_value);
 	  if (ret == CDADA_SUCCESS) {
 	    if ((*idx_value) > ptr->pos) {
 	      ptr->jeq.ptr = &t->e[(*idx_value)];
@@ -914,13 +938,23 @@ int pretag_copy_label(pt_label_t *dst, pt_label_t *src)
   else {
     if (src->len) {
       pretag_malloc_label(dst, src->len + 1);
+
       if (!dst->val) {
         Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (pretag_copy_label).\n", config.name, config.type);
         return ERR;
       }
 
       strncpy(dst->val, src->val, src->len);
-      dst->val[dst->len - 1] = '\0';
+
+      if (dst->val[src->len - 1] == '\0') {
+         /* fix length in case of multiple mallocs */
+        dst->len = src->len;
+      }
+      else {
+         /* add null char only when not already there */
+        dst->val[dst->len - 1] = '\0';
+      }    
+
     }
   }
   
@@ -1071,8 +1105,10 @@ pt_bitmap_t pretag_index_build_bitmap(struct id_entry *ptr, int acct_type)
   if (idx_bmap & PRETAG_SET_TAG2) idx_bmap ^= PRETAG_SET_TAG2;
   if (idx_bmap & PRETAG_SET_LABEL) idx_bmap ^= PRETAG_SET_LABEL;
 
-  /* 3) handle the case of catch-all rule */
-  if (!idx_bmap) idx_bmap = PRETAG_NULL;
+  /* 3) if 'ip' key is not defined, still define the address family (AF)
+     so to be memory-savvy and avoid creating duplicate entries in case
+     of v4/v6 recirculation */
+  if (!(idx_bmap & PRETAG_IP)) idx_bmap |= PRETAG_IP_AF;
 
   return idx_bmap;
 }
