@@ -71,6 +71,7 @@ extern pm_id_t PT_stack_sum(pm_id_t, pm_id_t);
 extern pm_id_t PT_stack_logical_or(pm_id_t, pm_id_t);
 
 extern int PT_map_index_entries_ip_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_ip_af_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
 extern int PT_map_index_entries_input_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
 extern int PT_map_index_entries_output_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
 extern int PT_map_index_entries_bgp_nexthop_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
@@ -90,8 +91,8 @@ extern int PT_map_index_entries_src_net_handler(struct id_table_index *, int, pm
 extern int PT_map_index_entries_dst_net_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
 extern int PT_map_index_entries_is_multicast_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
 extern int PT_map_index_entries_fwd_status_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
-extern int PT_map_index_entries_null_handler(struct id_table_index *, int, pm_hash_serial_t *, void *);
 extern int PT_map_index_fdata_ip_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_fdata_ip_af_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_fdata_input_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_fdata_output_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_fdata_bgp_nexthop_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *); 
@@ -111,7 +112,6 @@ extern int PT_map_index_fdata_src_net_handler(struct id_table_index *, int, int,
 extern int PT_map_index_fdata_dst_net_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
 extern int PT_map_index_fdata_is_multicast_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
 extern int PT_map_index_fdata_fwd_status_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
-extern int PT_map_index_fdata_null_handler(struct id_table_index *, int, int, struct id_entry *, pm_hash_serial_t *, void *);
 
 /* BPAS_*: bgp_peer_as_src map specific handlers */
 extern int BPAS_map_bgp_nexthop_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -187,6 +187,7 @@ extern int SF_pretag_dst_as_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_src_mac_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_dst_mac_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_vlan_id_handler(struct packet_ptrs *, void *, void *);
+extern int SF_pretag_cvlan_id_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_mpls_pw_id_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_src_net_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_dst_net_handler(struct packet_ptrs *, void *, void *);
