@@ -1,7 +1,7 @@
 /*  
  * pmacct (Promiscuous mode IP Accounting package)
  *
- * Copyright (c) 2003-2022 Paolo Lucente <paolo@pmacct.net>
+ * Copyright (c) 2003-2023 Paolo Lucente <paolo@pmacct.net>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -20,6 +20,7 @@
 #include "pmacct.h"
 #include "pmacct-data.h"
 #include "thread_pool.h"
+#include <sys/time.h>
 
 /* Global variables */
 thread_pool_t *redis_pool;
@@ -43,7 +44,7 @@ int p_redis_master_produce_thread(void *rh)
 {
   struct p_redis_host *redis_host = rh;
   unsigned int ret = 0, period = 0;
-
+  
   p_redis_connect(redis_host, TRUE);
 
   for (;;) {
@@ -84,37 +85,48 @@ void p_redis_init(struct p_redis_host *redis_host, char *log_id, redis_thread_ha
 
 int p_redis_connect(struct p_redis_host *redis_host, int fatal)
 {
-  struct sockaddr_storage dest;
-  socklen_t dest_len = sizeof(dest);
-  char dest_str[INET6_ADDRSTRLEN];
-  int dest_port;
-
-  time_t now = time(NULL);
-
   assert(redis_host);
 
   if (config.redis_host) {
+
+    time_t now = time(NULL);
+
     if (now >= (redis_host->last_conn + PM_REDIS_DEFAULT_CONN_RETRY)) {
+      int res = REDIS_OK;
       redis_host->last_conn = now;
+      if (redis_host->ctx) {
+        /* reconnect */
+        Log(LOG_DEBUG, "DEBUG ( %s ): reconnecting to redis server (fd %d)\n",
+            redis_host->log_id, redis_host->ctx->fd);
+        res = redisReconnect(redis_host->ctx);
+      }
+      else {
+        /* initial connect */
+        struct sockaddr_storage dest;
+        socklen_t dest_len = sizeof(dest);
+        char dest_str[INET6_ADDRSTRLEN];
+        int dest_port;
 
-      /* round of parsing and validation */
-      parse_hostport(config.redis_host, (struct sockaddr *)&dest, &dest_len);
-      sa_to_str(dest_str, sizeof(dest_str), (struct sockaddr *)&dest, FALSE);
-
-      sa_to_port(&dest_port, (struct sockaddr *)&dest);
-      if (!dest_port) {
-	dest_port = PM_REDIS_DEFAULT_PORT;
+        /* round of parsing and validation */
+        parse_hostport(config.redis_host, (struct sockaddr *)&dest, &dest_len);
+        sa_to_str(dest_str, sizeof(dest_str), (struct sockaddr *)&dest, FALSE);
+        sa_to_port(&dest_port, (struct sockaddr *)&dest);
+        if (!dest_port) {
+          dest_port = PM_REDIS_DEFAULT_PORT;
+        }
+        Log(LOG_INFO, "INFO ( %s ): connecting to redis server %s:%d\n",
+            redis_host->log_id, dest_str, dest_port);
+        redis_host->ctx = redisConnect(dest_str, dest_port);
       }
 
-      redis_host->ctx = redisConnect(dest_str, dest_port);
-
-      if (redis_host->ctx == NULL || redis_host->ctx->err) {
+      if (res != REDIS_OK || redis_host->ctx == NULL || redis_host->ctx->err) {
 	if (redis_host->ctx) {
 	  if (fatal) {
-	    Log(LOG_ERR, "ERROR ( %s ): Connection error: %s\n", redis_host->log_id, redis_host->ctx->errstr);
-	    exit_gracefully(1);
+      Log(LOG_ERR, "ERROR ( %s ): Can't connect to redis server: %s\n", redis_host->log_id, redis_host->ctx->errstr);
+      exit_gracefully(1);
 	  }
 	  else {
+      Log(LOG_WARNING, "WARN ( %s ): Can't connect to redis server: %s\n", redis_host->log_id, redis_host->ctx->errstr);
 	    return ERR;
 	  }
 	}
@@ -151,6 +163,34 @@ void p_redis_set_string(struct p_redis_host *redis_host, char *resource, char *v
   p_redis_process_reply(redis_host);
 }
 
+/* Get redis string-value based on key */
+void p_redis_get_string(struct p_redis_host *redis_host, char *resource, char *reply_string)
+{
+  redis_host->reply = redisCommand(redis_host->ctx, "GET %s", resource);
+  if(redis_host->reply) strncpy(reply_string, redis_host->reply->str, SHORTBUFLEN);
+
+  p_redis_process_reply(redis_host);
+}
+
+/* Get redis keys matching regex string pattern */
+void p_redis_get_keys(struct p_redis_host *redis_host, char *regex, struct p_redis_keys *redis_keys)
+{
+  redis_host->reply = redisCommand(redis_host->ctx, "KEYS %s", regex);
+
+  if (redis_host->reply) {
+    redis_keys->keys_amount = (int)redis_host->reply->elements;
+    if (redis_keys->keys_amount != 0)
+    {
+        for (int i = 0; i < redis_keys->keys_amount; i++)
+        {
+          strncpy(redis_keys->keys[i], redis_host->reply->element[i]->str, sizeof(redis_keys->keys[i]));
+        }
+    }
+  }
+
+  p_redis_process_reply(redis_host);
+}
+
 void p_redis_set_int(struct p_redis_host *redis_host, char *resource, int value, int expire)
 {
   if (expire > 0) {
@@ -289,3 +329,4 @@ void p_redis_thread_produce_common_plugin_handler(void *rh)
 	   config.name, PM_REDIS_DEFAULT_SEP, config.type);
   p_redis_set_int(redis_host, name_and_type, TRUE, PM_REDIS_DEFAULT_EXP_TIME);
 }
+
