@@ -34,20 +34,11 @@
 extern char *__progname;
 #endif
 
-char pmacctd_globstr[] = "pmacctd\0";
-char nfacctd_globstr[] = "nfacctd\0";
-char sfacctd_globstr[] = "sfacctd\0";
-char uacctd_globstr[] = "uacctd\0";
-char pmtele_globstr[] = "pmtelemetryd\0";
-char pmbgpd_globstr[] = "pmbgpd\0";
-char pmbmpd_globstr[] = "pmbmpd\0";
-
 /*
  * NEWSTR -- Create a copy of a C string
  */
 
-char *spt_newstr(s)
-const char *s;
+char *spt_newstr(const char *s)
 {
   size_t l;
   char *n;
@@ -89,13 +80,9 @@ static char	**Argv = NULL;		/* pointer to argument vector */
 static char	*LastArgv = NULL;	/* end of argv */
 
 void
-initsetproctitle(argc, argv, envp)
-	int argc;
-	char **argv;
-	char **envp;
+initsetproctitle(int argc, char **argv, char **envp)
 {
 	register int i;
-	int align;
 	extern char **environ;
 
 	/*
@@ -136,10 +123,10 @@ initsetproctitle(argc, argv, envp)
 	}
 
 #if defined PROGNAME && SPT_TYPE == SPT_REUSEARGV
-	if (config.acct_type == ACCT_PM) __progname = pmacctd_globstr;
+	if (config.uacctd_group) __progname = uacctd_globstr; /* XXX: hack */
+	else if (config.acct_type == ACCT_PM) __progname = pmacctd_globstr;
 	else if (config.acct_type == ACCT_NF) __progname = nfacctd_globstr;
 	else if (config.acct_type == ACCT_SF) __progname = sfacctd_globstr;
-	else if (config.acct_type == ACCT_UL) __progname = uacctd_globstr;
 	else if (config.acct_type == ACCT_PMTELE) __progname = pmtele_globstr;
 	else if (config.acct_type == ACCT_PMBGP) __progname = pmbgpd_globstr;
 	else if (config.acct_type == ACCT_PMBMP) __progname = pmbmpd_globstr;
