@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -181,6 +181,14 @@ void PM_sigint_handler(int signum)
     }
   }
 
+  if (config.acct_type == ACCT_PMTELE) {
+    if (config.telemetry_grpc_collector_conf) {
+#if defined WITH_ZMQ
+      p_zmq_remove_ipc_file(config.telemetry_grpc_collector_socket);
+#endif
+    }
+  }
+
   if (config.pidfile) remove_pid_file(config.pidfile);
 
   if (config.propagate_signals) signal_kittens(signum, TRUE);
@@ -222,6 +230,7 @@ void reload_maps(int signum)
   reload_map = FALSE;
   reload_map_bgp_thread = FALSE;
   reload_map_bmp_thread = FALSE;
+  reload_map_telemetry_thread = FALSE;
   reload_map_rpki_thread = FALSE;
   reload_map_exec_plugins = FALSE;
   reload_geoipv2_file = FALSE;
@@ -230,6 +239,7 @@ void reload_maps(int signum)
     reload_map = TRUE; 
     reload_map_bgp_thread = TRUE;
     reload_map_bmp_thread = TRUE;
+    reload_map_telemetry_thread = TRUE;
     reload_map_rpki_thread = TRUE;
     reload_map_exec_plugins = TRUE;
     reload_geoipv2_file = TRUE;
