@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -193,6 +193,8 @@ extern void count_src_mac_handler(const struct db_cache *, struct insert_data *,
 extern void count_dst_mac_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_vlan_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_out_vlan_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_in_cvlan_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_out_cvlan_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_cos_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_etype_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_src_host_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
@@ -246,9 +248,13 @@ extern void count_post_nat_dst_port_handler(const struct db_cache *, struct inse
 extern void count_nat_event_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_fw_event_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_fwd_status_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_flow_label_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_mpls_label_top_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_mpls_label_bottom_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_mpls_label_stack_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_path_delay_avg_usec_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_path_delay_min_usec_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_path_delay_max_usec_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_tunnel_src_mac_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_tunnel_dst_mac_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_tunnel_src_ip_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
@@ -259,7 +265,9 @@ extern void count_tunnel_ip_tos_handler(const struct db_cache *, struct insert_d
 extern void count_tunnel_src_port_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_tunnel_dst_port_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_tunnel_tcpflags_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_tunnel_flow_label_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_vxlan_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
+extern void count_nvgre_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_timestamp_start_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void PG_copy_count_timestamp_start_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
 extern void count_timestamp_start_residual_handler(const struct db_cache *, struct insert_data *, int, char **, char **);
@@ -384,6 +392,7 @@ extern char insert_full_clause[LONGSRVBUFLEN];
 extern char values_clause[LONGLONGSRVBUFLEN];
 extern char *multi_values_buffer;
 extern char where_clause[LONGLONGSRVBUFLEN];
+extern char timezone_clause[LONGSRVBUFLEN];
 extern unsigned char *pipebuf;
 extern struct db_cache *sql_cache;
 extern struct db_cache **sql_queries_queue, **sql_pending_queries_queue;
