@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2023 by Paolo Lucente
 */
 
 /*
@@ -36,6 +36,7 @@ char sqlite3_table_v5[] = "acct_v5";
 char sqlite3_table_v6[] = "acct_v6";
 char sqlite3_table_v7[] = "acct_v7";
 char sqlite3_table_v8[] = "acct_v8";
+char sqlite3_table_v9[] = "acct_v9";
 char sqlite3_table_bgp[] = "acct_bgp";
 
 /* Functions */
@@ -128,6 +129,12 @@ void sqlite3_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   }
 #endif
 
+  if (config.dry_run == DRY_RUN_SETUP) {
+    sleep(DEFAULT_SLOTH_SLEEP_TIME); /* Make sure all comes up */
+    printf("INFO ( %s/%s ): Dry run 'setup'. Exiting ..\n", config.name, config.type);
+    exit(0);
+ }
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -747,6 +754,7 @@ void SQLI_init_default_values(struct insert_data *idata)
   if (!config.sql_db) config.sql_db = sqlite3_db;
   if (!config.sql_table) {
     if (config.sql_table_version == (SQL_TABLE_VERSION_BGP+1)) config.sql_table = sqlite3_table_bgp;
+    else if (config.sql_table_version == 9) config.sql_table = sqlite3_table_v9;
     else if (config.sql_table_version == 8) config.sql_table = sqlite3_table_v8;
     else if (config.sql_table_version == 7) config.sql_table = sqlite3_table_v7;
     else if (config.sql_table_version == 6) config.sql_table = sqlite3_table_v6;
