@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2024 by Paolo Lucente
 */
 
 /*
@@ -21,6 +21,7 @@
 
 /* includes */
 #include "pmacct.h"
+#include "pmacct-build.h"
 #include "util-data.h"
 #ifdef WITH_KAFKA
 #include "kafka_common.h"
@@ -351,10 +352,10 @@ void string_add_newline(char *buf)
 
 time_t roundoff_time(time_t t, char *value)
 {
-  struct tm *rounded;
+  struct tm result_tm, *rounded;
   int len, j;
 
-  rounded = localtime(&t);
+  rounded = localtime_r(&t, &result_tm);
 
   rounded->tm_sec = 0; /* default round off */
 
@@ -374,7 +375,7 @@ time_t roundoff_time(time_t t, char *value)
       else if (value[j] == 'w') {
         rounded->tm_min = 0;
         rounded->tm_hour = 0;
-	while (rounded->tm_wday > 1) {
+	while (rounded->tm_wday >= 1) {
 	  rounded->tm_mday--;
 	  rounded->tm_wday--;
 	}
@@ -397,10 +398,14 @@ time_t roundoff_time(time_t t, char *value)
 time_t calc_monthly_timeslot(time_t t, int howmany, int op)
 {
   time_t base = t, final;
-  struct tm *tmt;
+  struct tm result_tm, *tmt;
 
-  if (!config.timestamps_utc) tmt = localtime(&t);
-  else tmt = gmtime(&t);
+  if (!config.timestamps_utc) {
+    tmt = localtime_r(&t, &result_tm);
+  }
+  else {
+    tmt = gmtime_r(&t, &result_tm);
+  }
 
   while (howmany) {
     tmt->tm_mday = 1;
@@ -1258,13 +1263,21 @@ void write_pid_file(char *filename)
   char pid[10];
   uid_t owner = -1;
   gid_t group = -1;
+  int ret;
 
   unlink(filename); 
 
   if (config.files_uid) owner = config.files_uid;
   if (config.files_gid) group = config.files_gid;
+
+  /* create dir structure to get to file, if needed */
+  ret = mkdir_multilevel(filename, TRUE, owner, group);
+  if (ret) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): [%s] write_pid_file(): mkdir_multilevel() failed.\n", config.name, config.type, filename);
+    return;
+  }
     
-  file = fopen(filename,"w");
+  file = fopen(filename, "w");
   if (file) {
     if (chown(filename, owner, group) == -1)
       Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Unable to chown(): %s\n", config.name, config.type, filename, strerror(errno));
@@ -1287,11 +1300,13 @@ void write_pid_file(char *filename)
 
 void write_pid_file_plugin(char *filename, char *type, char *name)
 {
+  
   int len = strlen(filename) + strlen(type) + strlen(name) + 3;
   FILE *file;
   char *fname, pid[10], minus[] = "-";
   uid_t owner = -1;
   gid_t group = -1;
+  int ret;
 
   fname = malloc(len);
   if (!fname) {
@@ -1311,6 +1326,13 @@ void write_pid_file_plugin(char *filename, char *type, char *name)
   if (config.files_uid) owner = config.files_uid;
   if (config.files_gid) group = config.files_gid;
 
+  /* create dir structure to get to file, if needed */
+  ret = mkdir_multilevel(fname, TRUE, owner, group);
+  if (ret) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): [%s] write_pid_file_plugin(): mkdir_multilevel() failed.\n", config.name, config.type, fname);
+    return;
+  }
+
   file = fopen(fname, "w");
   if (file) {
     if (chown(fname, owner, group) == -1)
@@ -1498,7 +1520,7 @@ void lower_string(char *string)
   }
 }
 
-void evaluate_sums(u_int64_t *wtc, u_int64_t *wtc_2, char *name, char *type)
+void evaluate_sums(u_int64_t *wtc, u_int64_t *wtc_2, u_int64_t *wtc_3, char *name, char *type)
 {
   int tag = FALSE;
   int tag2 = FALSE;
@@ -1578,13 +1600,17 @@ void stop_all_childs()
 void pm_strftime(char *s, int max, char *format, const time_t *time_ref, int utc)
 {
   time_t time_loc;  
-  struct tm *tm_loc;
+  struct tm result_tm, *tm_loc;
 
   if (time_ref && (*time_ref)) time_loc = (*time_ref);
   else time_loc = time(NULL);
 
-  if (!utc) tm_loc = localtime(&time_loc);
-  else tm_loc = gmtime(&time_loc);
+  if (!utc) {
+    tm_loc = localtime_r(&time_loc, &result_tm);
+  }
+  else {
+    tm_loc = gmtime_r(&time_loc, &result_tm);
+  }
 
   strftime(s, max, format, tm_loc);
   insert_rfc3339_timezone(s, max, tm_loc);
@@ -1595,13 +1621,17 @@ void pm_strftime(char *s, int max, char *format, const time_t *time_ref, int utc
 void pm_strftime_same(char *s, int max, char *tmp, const time_t *time_ref, int utc)
 {
   time_t time_loc;
-  struct tm *tm_loc;
+  struct tm result_tm, *tm_loc;
 
   if (time_ref && (*time_ref)) time_loc = (*time_ref);
   else time_loc = time(NULL);
 
-  if (!utc) tm_loc = localtime(&time_loc);
-  else tm_loc = gmtime(&time_loc);
+  if (!utc) {
+    tm_loc = localtime_r(&time_loc, &result_tm);
+  }
+  else {
+    tm_loc = gmtime_r(&time_loc, &result_tm);
+  }
 
   strftime(tmp, max, s, tm_loc);
   insert_rfc3339_timezone(tmp, max, tm_loc);
@@ -1912,7 +1942,7 @@ void set_default_preferences(struct configuration *cfg)
   if (config.acct_type == ACCT_NF || config.acct_type == ACCT_SF) {
     if (!cfg->nfacctd_net) cfg->nfacctd_net = NF_NET_KEEP;
     if (!cfg->nfacctd_as) cfg->nfacctd_as = NF_AS_KEEP;
-    set_truefalse_nonzero(&cfg->nfacctd_disable_checks);
+    set_truefalse_nonzero(&cfg->nfacctd_disable_sanity_checks);
   }
   if (!cfg->bgp_daemon_peer_as_src_type) cfg->bgp_daemon_peer_as_src_type = BGP_SRC_PRIMITIVES_KEEP;
   if (!cfg->bgp_daemon_src_std_comm_type) cfg->bgp_daemon_src_std_comm_type = BGP_SRC_PRIMITIVES_KEEP;
@@ -2280,6 +2310,9 @@ void version_daemon(int acct_type, char *header)
 #ifdef WITH_UNYTE_UDP_NOTIF
   printf("unyte-udp-notif\n");
 #endif
+#ifdef WITH_GRPC_COLLECTOR
+  printf("mdt-dialout-collector\n");
+#endif
 #ifdef WITH_NDPI
   printf("nDPI %s\n", ndpi_revision());
 #endif
@@ -2417,7 +2450,7 @@ void compose_timestamp(char *buf, int buflen, struct timeval *tv, int usec, int
 {
   int slen;
   time_t time1;
-  struct tm *time2;
+  struct tm result_tm, *time2;
 
   if (buflen < VERYSHORTBUFLEN) return; 
 
@@ -2427,8 +2460,12 @@ void compose_timestamp(char *buf, int buflen, struct timeval *tv, int usec, int
   }
   else {
     time1 = tv->tv_sec;
-    if (!utc) time2 = localtime(&time1);
-    else time2 = gmtime(&time1);
+    if (!utc) {
+      time2 = localtime_r(&time1, &result_tm);
+    }
+    else {
+      time2 = gmtime_r(&time1, &result_tm);
+    }
     
     if (tv->tv_sec) {
       if (!rfc3339) slen = strftime(buf, buflen, "%Y-%m-%d %H:%M:%S", time2);
@@ -3475,10 +3512,13 @@ void parse_hostport(const char *s, struct sockaddr *addr, socklen_t *len)
     Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), invalid '%s' argument\n", config.name, config.type, orig);
     exit_gracefully(1);
   }
-  *(port - 1) = '\0';
+
+  if (port) { 
+    *(port - 1) = '\0';
+  }
 	
   /* Accept [host]:port for numeric IPv6 addresses */
-  if (*host == '[' && *(port - 2) == ']') {
+  if (host && port && *host == '[' && *(port - 2) == ']') {
     host++;
     *(port - 2) = '\0';
   }
@@ -3754,3 +3794,21 @@ u_int8_t sampling_direction_str2id(char *sd_str)
 
   return SAMPLING_DIRECTION_UNKNOWN;
 }
+
+void remove_password(char *str, char *pwdstr)
+{
+  char *ptr = NULL;
+
+  ptr = strstr(str, pwdstr);
+  if (ptr) {
+    ptr += strlen(pwdstr);
+
+    if ((*ptr) == '=') {
+      ptr++;
+      while (!isblank((*ptr)) && !((*ptr) == '\0')) {
+        (*ptr) = 'X';
+	ptr++;
+      }
+    }
+  }
+}
