@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2022 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2024 by Paolo Lucente
 */
 
 /*
@@ -81,7 +81,7 @@ extern void mark_columns(char *);
 extern int Setsocksize(int, int, int, void *, socklen_t);
 extern void *map_shared(void *, size_t, int, int, int, off_t);
 extern void lower_string(char *);
-extern void evaluate_sums(u_int64_t *, u_int64_t *, char *, char *);
+extern void evaluate_sums(u_int64_t *, u_int64_t *, u_int64_t *, char *, char *);
 extern void stop_all_childs();
 extern int file_lock(int);
 extern int file_unlock(int);
@@ -140,6 +140,7 @@ extern void reload_logs(char *);
 extern int is_empty_256b(void *, int);
 extern ssize_t pm_recv(int, void *, size_t, int, unsigned int);
 extern int ft2af(u_int8_t);
+extern void remove_password(char *, char *);
 
 extern char *compose_json_str(void *);
 extern void write_and_free_json(FILE *, void *);
